/**
 * 
 */
package de.cesr.sonomode.agent;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.random.RandomHelper;
import cern.jet.random.Uniform;
import de.cesr.more.basic.agent.MAbstractAnalyseNetworkAgent;
import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.manipulate.agent.MoreEgoNetworkManagerComp;
import de.cesr.more.manipulate.agent.events.MLinkManagingEvent;
import de.cesr.more.manipulate.agent.events.MLinkWeightUpdatingEvent;
import de.cesr.more.measures.MMeasureDescription;
import de.cesr.more.measures.node.MNodeMeasures;
import de.cesr.more.measures.node.MoreNodeMeasureSupport;
import de.cesr.more.param.MMilieuNetworkParameterMap;
import de.cesr.more.param.MNetworkBuildingPa;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sesamgim.context.GimGroupContext;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.function.KernelFunction;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.param.MilieuPa;
import de.cesr.sonomode.param.RandomPa;
import de.cesr.sonomode.socnet.SonomodeEdge;
import de.cesr.sonomode.util.MilieuPrefParameterMap;

/**
 * @author holzhauer
 *
 */
public abstract class AbstractSonomodeAgent extends MAbstractAnalyseNetworkAgent<SonomodeAgent, 
 SonomodeEdge> implements
		SonomodeAgent {
	
	static final double PRECISION = 0.0000001;
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(AbstractSonomodeAgent.class);
	static int currentID = 1;
	
	// static
	protected Logger alogger;
	
	double persUncertainty;
	double persOpinion;
	
	double tmpUncertainty;
	double tmpOpinion;
	
	double convergenceTH;
	
	double finalRadius;
	int numRadiusExtensions;

	int numNewLinksTick = 0;
	int numNewLinksTotal = 0;
	int numNewTransitiveLinksTotal = 0;
	int numNewReciprocalLinksTotal = 0;
	int numNewLocalLinksTotal = 0;

	int numPotTransitiveLinksTotal = 0;
	int numPotReciprocalLinksTotal = 0;
	int numPotLocalLinksTotal = 0;
	
	double tickUpdatedNewLinksTick = Double.NEGATIVE_INFINITY;

	int extremismGroup;
	int sinusMilieuGroup;
	
	double convergenceTick = Double.MIN_VALUE;
	
	boolean updated = false;
	
	KernelFunction function = null;

	int id;
	
	protected GimGroupContext<SonomodeAgent> gc;

	protected MoreEgoNetworkManagerComp<SonomodeAgent, SonomodeEdge> egoNetComp;

	/**
	 * @return the id
	 */
	public int getId() {
		return id;
	}

	Uniform uniformNoise;
	Uniform uniformNoiseAmount;
	

	public AbstractSonomodeAgent(double opinion, int milieuGroup, int sinusMilieuGroup) {
		this.extremismGroup = milieuGroup;
		this.sinusMilieuGroup = sinusMilieuGroup;
		this.persOpinion = opinion;
		this.id = currentID++;
		this.persUncertainty = ((Double)((MilieuPrefParameterMap)PmParameterManager.getParameter(MilieuPa.MILIEU_PREF_PARAMS)).getPref(getXtremismGp(), MilieuPa.UNCERTAINTY)).doubleValue();
		this.uniformNoise = (Uniform) RandomHelper.getDistribution(ModelManager.RANDOM_NOISE);
		this.uniformNoiseAmount = (Uniform) RandomHelper.getDistribution(ModelManager.RANDOM_NOISE_AMOUNT);
		this.convergenceTH = ((Double) PmParameterManager.getParameter(BasicPa.AGENT_CONVERGENCE_THRESHOLD)).doubleValue();

		alogger = Logger.getLogger(AbstractSonomodeAgent.class
				.getName() + "." + this.toString());
	}
	
	@Override
	public double updateProperties(double oldPartnerOpinion, double oldPartnerUncertainty, SonomodeAgent partner) {

		updated = true;

		if (function == null) {
			try {
				try {
					function = (KernelFunction) ((Class<?>) PmParameterManager.getParameter(BasicPa.KERNEL_FUNCTION)).getMethod("getInstance").invoke(null);
				} catch (NoSuchMethodException e) {
		
					function = (KernelFunction) ((Class<?>) PmParameterManager.getParameter(BasicPa.KERNEL_FUNCTION)).
							getConstructor().newInstance();
				}
				
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InstantiationException e1) {
				e1.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		
		// TODO its seems as oldOwnX would not be requires (use persX instead?!)
		double oldOwnOpinion = persOpinion;
		double oldOwnUncertainty = persUncertainty;

		double mue = ((Double)((MilieuPrefParameterMap)PmParameterManager.getParameter(MilieuPa.MILIEU_PREF_PARAMS)).getPref(getXtremismGp(), MilieuPa.MUE)).doubleValue();
		double kernel = function.getValue(this.persOpinion, oldPartnerOpinion, this.persUncertainty, oldPartnerUncertainty);
		
		if ((Boolean) PmParameterManager.getParameter(BasicPa.ASSYNCHRON_POP_UPDATE) && (Boolean) PmParameterManager.getParameter(BasicPa.ASSYNCHRON_UPDATE)) {
			// assynchron
			// TODO
			this.tmpOpinion = this.persOpinion + mue * kernel * (partner.getOpinion() - this.persOpinion);
			this.tmpUncertainty = this.persUncertainty + mue * kernel * (partner.getUncertaint() - this.persUncertainty);
		} else {
			//synchron
			this.tmpOpinion = this.persOpinion + mue * kernel * (oldPartnerOpinion - this.persOpinion);
			this.tmpUncertainty = this.persUncertainty + mue * kernel * (oldPartnerUncertainty - this.persUncertainty);
		}
		
		if ((Boolean) PmParameterManager.getParameter(BasicPa.ASSYNCHRON_POP_UPDATE)) {
			this.setPersistentVariables();
		}

		// <- LOGGING
		if (alogger.isDebugEnabled()) {
			if (Math.abs(oldOwnOpinion - this.tmpOpinion) > PRECISION) {
				alogger.debug(this
						+ "> Opinion change: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldOwnOpinion)
						+ " -> "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(this.tmpOpinion)
						+ " (unvertainty: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldOwnUncertainty)
						+ " -> "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(this.tmpUncertainty)
						+ " [partner ("
						+ partner
						+ ") "
						+ " Old Opinion: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldPartnerOpinion)
						+ " / Old Unc.: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldPartnerUncertainty)
						+ "] :: mue: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(mue)
						+ " :: Kernel function: "
						+ ModelManager
								.getMan()
								.getFloatPointFormat()
								.format(function.getValue(oldOwnOpinion,
										oldPartnerOpinion, oldOwnUncertainty,
										oldPartnerUncertainty)));
			} else {
				alogger.debug(this
						+ "> No opinion change ("
						+ this.tmpOpinion
						+ ") with "
						+ partner
						+ " (Opinion: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldPartnerOpinion)
						+ " / Unc.: "
						+ ModelManager.getMan().getFloatPointFormat()
								.format(oldPartnerUncertainty) + ")");
			}
		}
		// LOGGING ->
			
		return Math.abs(oldOwnOpinion - this.tmpOpinion);
	}
	
	@Override
	public void initDynamicNetworkCode() {

		MMilieuNetworkParameterMap milieuNetPrefs = (MMilieuNetworkParameterMap) PmParameterManager
				.getParameter(MNetworkBuildingPa.MILIEU_NETWORK_PARAMS);

		RunEnvironment.getInstance().getCurrentSchedule().schedule(this);

		if (milieuNetPrefs.getDynEdgeUpdatingInverval(getMilieuGroup()) > 0) {
			RunEnvironment
					.getInstance()
					.getCurrentSchedule()
					.schedule(
							ScheduleParameters.createOneTime(
									new Double(
											milieuNetPrefs
													.getDynEdgeUpdatingInverval(getMilieuGroup()))
											* (-1), Double.MAX_VALUE),
							this,
							"updateLinkWeights");
		} else if (milieuNetPrefs.getDynEdgeUpdatingInverval(getMilieuGroup()) < 0) {
			RunEnvironment
					.getInstance()
					.getCurrentSchedule()
					.schedule(
							ScheduleParameters.createRepeating(
									1,
									new Double(
											milieuNetPrefs
													.getDynEdgeUpdatingInverval(getMilieuGroup()))
											* (-1), Double.MAX_VALUE),
							this,
							"updateLinkWeights");
		}

		if (milieuNetPrefs.getDynEdgeUpdatingInverval(getMilieuGroup()) > 0) {
			RunEnvironment
					.getInstance()
					.getCurrentSchedule()
					.schedule(
							ScheduleParameters.createOneTime(
									new Double(
											milieuNetPrefs
													.getDynLinkManagementInverval(getMilieuGroup())),
													Double.MAX_VALUE - 1), this,
							"manageEdges");
		} else if (milieuNetPrefs.getDynEdgeUpdatingInverval(getMilieuGroup()) < 0) {
			RunEnvironment
					.getInstance()
					.getCurrentSchedule()
					.schedule(
							ScheduleParameters.createRepeating(
									1,
									new Double(
											milieuNetPrefs
													.getDynLinkManagementInverval(getMilieuGroup()))
											* (-1), Double.MAX_VALUE - 1),
							this,
							"manageEdges");
		}
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#setPersistentVariables()
	 */
	@Override
	public void setPersistentVariables() {

		if (updated) {
			// check for convergence:
			if (Math.abs(persOpinion - this.tmpOpinion) <= convergenceTH) {
				if (this.convergenceTick == Double.MIN_VALUE) {
					this.convergenceTick = RunEnvironment.getInstance()
							.getCurrentSchedule().getTickCount();
				}
			} else {
				if (this.convergenceTick > Double.MIN_VALUE) {
					this.convergenceTick = Double.MIN_VALUE;
				}
			}

			this.persUncertainty = this.tmpUncertainty;
			this.persOpinion = this.tmpOpinion;
			updated = false;
		}
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#addNoise()
	 */
	@Override
	public void addNoise() {
		if (uniformNoise.nextDouble() <= ((Double)PmParameterManager.getParameter(RandomPa.NOISE_TRESHOLD)).doubleValue()) {
			double noise = uniformNoiseAmount.nextDouble();
			this.persOpinion = this.persOpinion + noise * this.getUncertaint();
			
			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("> " + this + " added noise: " + ModelManager.getMan().getFloatPointFormat().format(noise * this.getUncertaint()));
			}
			// LOGGING ->
		}
	}
	
	/**
	 * 
	 */
	public static void reset() {
		currentID = 1;
	}
	
	/**********************************************************
	 *** Network Measure Support ***
	 **********************************************************/

	MNodeMeasures				measures	= new MNodeMeasures();
	
	/**
	 * @see de.cesr.more.measures.node.MoreNodeMeasureSupport#getNetworkMeasureObject(de.cesr.more.basic.network.MoreNetwork,
	 * de.cesr.more.measures.MMeasureDescription)
	 */
	@Override
	public Number getNetworkMeasureObject(MoreNetwork<? extends MoreNodeMeasureSupport, ?> network,
			MMeasureDescription key) {

		if (measures.getNetworkMeasureObject(network, key) == null) {
			// <- LOGGING
			logger.error("No mesure defined for key " + key);
			// LOGGING ->
		}
		return measures.getNetworkMeasureObject(network, key);
	}

	/**
	 * @see de.cesr.more.measures.node.MoreNodeMeasureSupport#setNetworkMeasureObject(de.cesr.more.basic.network.MoreNetwork,
	 * de.cesr.more.measures.MMeasureDescription, java.lang.Number)
	 */
	@Override
	public void setNetworkMeasureObject(MoreNetwork<? extends MoreNodeMeasureSupport, ?> network,
			MMeasureDescription key, Number value) {
		measures.setNetworkMeasureObject(network, key, value);
	}

	public void updateLinkWeights() {
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug(this + "> Update links...");
		}
		// LOGGING ->
		getEgoNetworkManager().process(MLinkWeightUpdatingEvent.getInstance(),
				this, getMainNetwork());
	}

	public void manageEdges() {
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug(this + "> Manage links...");
		}
		// LOGGING ->
		getEgoNetworkManager().process(MLinkManagingEvent.getInstance(), this,
				getMainNetwork());
	}


	@Override
	public AbstractSonomodeAgent getThis() {
		return this;
	}
	
	/**********************************************************
	 *** Overridden Methods 							    ***
	 **********************************************************/
	
	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return ModelManager.getMan().getIntegerFormat().format(this.id) + " ("
				+ getMilieuGroup() + ")";
	}
	
	/**
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return getId();
	}
	
	/**********************************************************
	 *** GETTER & SETTER                                    ***
	 **********************************************************/
	
	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getXtremismGp()
	 */
	@Override
	public int getXtremismGp() {
		return this.extremismGroup;
	}
	
	/**
	 * @see de.cesr.more.rs.building.MoreMilieuAgent#getAgentId()
	 */
	@Override
	public String getAgentId() {
		return new Integer(getId()).toString();
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getMilieuGroup()
	 */
	@Override
	public int getMilieuGroup() {
		return this.sinusMilieuGroup;
	}

	/**
	 * @param uncertainty
	 */
	@Override
	public void setUncertainty(double uncertainty) {
		this.persUncertainty = uncertainty;
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getUncertaint()
	 */
	@Override
	public double getUncertaint() {
		return persUncertainty;
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#setOpinion(double)
	 */
	@Override
	public void setOpinion(double opinion) {
		this.persOpinion = opinion;
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getOpinion()
	 */
	@Override
	public double getOpinion() {
		return persOpinion;
	}
	

	/**
	 * @see de.cesr.sesamgim.init.agent.GimMilieuAgent#getGroupContext()
	 */
	public GimGroupContext<SonomodeAgent> getGroupContext() {
		return this.gc;
	}
	

	@Override
	public Context<?> getParentContext() {
		return this.gc;
	}

	/**
	 * @see de.cesr.sesamgim.init.agent.GimMilieuAgent#setGroupContext(de.cesr.sesamgim.context.GGroupContext)
	 */
	public void setGroupContext(GimGroupContext<SonomodeAgent> gc) {
		this.gc = gc;
	}
	
	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getConverTick()
	 */
	@Override
	public float getConverTick() {
		return (float)this.convergenceTick;
	}

	@Override
	public void setNumRadiusExtensions(int num) {
		this.numRadiusExtensions = num;
	}

	@Override
	public int getNumRadiusExtensions() {
		return this.numRadiusExtensions;
	}

	@Override
	public void setFinalRadius(double finalRadius) {
		this.finalRadius = finalRadius;
	}

	@Override
	public double getFinalRadius() {
		return this.finalRadius;
	}

	/**
	 * @see de.cesr.more.basic.agent.MoreEgoNetworkManagingAgent#getEgoNetworkManager()
	 */
	@Override
	public MoreEgoNetworkManagerComp<SonomodeAgent, SonomodeEdge> getEgoNetworkManager() {
		return this.egoNetComp;
	}

	/**
	 * @see de.cesr.more.basic.agent.MoreEgoNetworkManagingAgent#setEgoNetworkManager(de.cesr.more.manipulate.agent.MoreEgoNetworkManagerComp)
	 */
	@Override
	public void setEgoNetworkManager(
			MoreEgoNetworkManagerComp<SonomodeAgent, SonomodeEdge> egoNetworkManager) {
		this.egoNetComp = egoNetworkManager;
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getNumNewLinks()
	 */
	@Override
	public int getNumNewLinks() {
		if (this.tickUpdatedNewLinksTick == RunEnvironment.getInstance()
				.getCurrentSchedule().getTickCount()) {
			return this.numNewLinksTick;
		} else {
			return 0;
		}
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getTotNewLinks()
	 */
	@Override
	public int getTotNewLinks() {
		return this.numNewLinksTotal;
	}

	@Override
	public int getTotNewTransitiveLinks() {
		return this.numNewTransitiveLinksTotal;
	}

	@Override
	public int getTotNewReciprocalLinks() {
		return this.numNewReciprocalLinksTotal;
	}

	@Override
	public int getTotNewLocalLinks() {
		return this.numNewLocalLinksTotal;
	}

	@Override
	public int getTotPotTransitiveLinks() {
		return this.numPotTransitiveLinksTotal;
	}

	@Override
	public int getTotPotReciprocalLinks() {
		return this.numPotReciprocalLinksTotal;
	}

	@Override
	public int getTotPotLocalLinks() {
		return this.numPotLocalLinksTotal;
	}
	
	@Override
	public void setNumPotTransitiveLinks(int counter) {
		this.numPotTransitiveLinksTotal +=counter;
	}

	@Override
	public void setNumPotReciprocalLinks(int counter) {
		this.numPotReciprocalLinksTotal +=counter;
	}

	@Override
	public void setNumPotLocalLinks(int counter) {
		this.numPotLocalLinksTotal +=counter;
	}
	
	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#setNumNewLinks(int)
	 */
	@Override
	public void setNumNewLinks(int numNewLinks) {
		this.tickUpdatedNewLinksTick = RunEnvironment.getInstance()
				.getCurrentSchedule().getTickCount();
		this.numNewLinksTick = numNewLinks;
		this.numNewLinksTotal += numNewLinks;
	}

	@Override
	public void setNumNewTransitiveLinks(int counter) {
		this.numNewTransitiveLinksTotal += counter;
	}

	@Override
	public void setNumNewReciprocalLinks(int counter) {
		this.numNewReciprocalLinksTotal += counter;
	}

	@Override
	public void setNumNewLocalLinks(int counter) {
		this.numNewLocalLinksTotal += counter;
	}

	/**
	 * @see de.cesr.sonomode.agent.SonomodeAgent#getAvgNNOpinionDev()
	 */
	@Override
	public double getAvgNNOpinionDev() {
		double sum = 0.0;
		for (SonomodeAgent neighbour : this.getMainNetwork().getPredecessors(
				this)) {
			sum += Math.abs(this.getOpinion() - neighbour.getOpinion());
		}
		return sum / this.getMainNetwork().getInDegree(this);
	}
	
	/**
	 * @see de.cesr.more.manipulate.agent.MoreLinkManipulatableAgent#getValueDifference(java.lang.Object)
	 */
	@Override
	public double getValueDifference(SonomodeAgent agent) {
		return this.getOpinion() - agent.getOpinion();
	}
}
