/**
 * 
 */
package de.cesr.sonomode.agent;

import de.cesr.more.basic.agent.MoreAgentAnalyseNetworkComp;
import de.cesr.more.basic.agent.MoreEgoNetworkManagingAgent;
import de.cesr.more.basic.agent.MoreNetworkAgent;
import de.cesr.more.manipulate.agent.MoreLinkManipulatableAgent;
import de.cesr.more.manipulate.agent.analyse.MoreLinkManipulationAnalysableAgent;
import de.cesr.more.rs.building.MoreDistanceAttachableAgent;
import de.cesr.more.rs.building.MoreMilieuAgent;
import de.cesr.more.rs.building.analyse.MoreBaselineNetworkServiceAnalysableAgent;
import de.cesr.sesamgim.init.agent.GimMilieuAgent;
import de.cesr.sonomode.socnet.SonomodeEdge;

/**
 * @author holzhauer
 *
 */
public interface SonomodeAgent extends MoreMilieuAgent, GimMilieuAgent<SonomodeAgent>, 
		MoreAgentAnalyseNetworkComp<SonomodeAgent, SonomodeEdge>,
		MoreNetworkAgent<SonomodeAgent, SonomodeEdge>,
		MoreDistanceAttachableAgent, MoreBaselineNetworkServiceAnalysableAgent,
		MoreEgoNetworkManagingAgent<SonomodeAgent, SonomodeEdge>,
		MoreLinkManipulatableAgent<SonomodeAgent>,
		MoreLinkManipulationAnalysableAgent{

	/**
	 * Sets the uncertainty
	 */
	public void setUncertainty(double uncertainty);

	/**
	 * @return the uncertainty
	 */
	public double getUncertaint();

	/**
	 * Sets the opinion
	 */
	public void setOpinion(double opinion);

	/**
	 * @return the opinion
	 */
	public double getOpinion();

	
	/**
	 * @param partnerOpinion
	 * @param partnerUncertainty
	 * @param partner
	 * @return the change in opinion
	 */
	public double updateProperties(double partnerOpinion, double partnerUncertainty, SonomodeAgent partner);
	
	/**
	 * For synchronous agent triggering, the temporary variables are copied to persistent ones. 
	 */
	public void setPersistentVariables();
	
	/**
	 * Adds noise to the agents opinion with probability RandomPa.NOISE_TRESHOLD.
	 */
	public void addNoise();
	
	public int getXtremismGp();
	
	@Override
	public int getMilieuGroup();
	
	public boolean isModerate();
	
	public float getConverTick();

	/**
	 * Number of links that were created in the current tick. NOTE: The result
	 * depends on the timing of calling setNumNewLinks()!
	 * 
	 * @return number of new ticks of a tick
	 */
	public int getNumNewLinks();

	/**
	 * Get the total number of new links for this agents. NOTE: Requires that
	 * setNumNewLinks() is called once per tick!
	 * 
	 * @return total number of new ticks
	 */
	public int getTotNewLinks();

	public int getTotNewTransitiveLinks();

	public int getTotNewReciprocalLinks();

	public int getTotNewLocalLinks();
	
	public int getTotPotTransitiveLinks();

	public int getTotPotReciprocalLinks();

	public int getTotPotLocalLinks() ;
	
	/**
	 * Set the number of new ticks during this tick.
	 * 
	 * @param numNewLinks
	 */
	@Override
	public void setNumNewLinks(int numNewLinks);

	/**
	 * Calculates the average opinion difference between ego and its
	 * predecessors.
	 * 
	 * @return the average opinion difference between ego and its predecessors.
	 */
	public double getAvgNNOpinionDev();
	
	public void initDynamicNetworkCode();
}
