/**
 * 
 */
package de.cesr.sonomode.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sesamgim.context.GAreaContext;
import de.cesr.sesamgim.init.area.GimAreaInitialiseService;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.core.ContextCreator;
import de.cesr.sonomode.param.BasicPa;

/**
 * @author Sascha Holzhauer
 *
 */
public class AreaContext extends GAreaContext<SonomodeAgent> {

	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(AreaContext.class);

	private final Map<Integer, Double> xLastMap = new HashMap<Integer, Double>();
	protected double convergenceTick = Double.NaN;
	
	static protected int id = 0;
	protected int name;

	public static class AreaContextIS implements
			GimAreaInitialiseService<SonomodeAgent> {

		@Override
		public Class<? extends GAreaContext<SonomodeAgent>> getAreaContextClass(
				Context<Object> superContext) {
			return AreaContext.class;
		}

		@Override
		public GAreaContext<SonomodeAgent> initArea(Context<Object> superContext) {
			return new AreaContext(superContext);
		}

	}

	public AreaContext(Context<Object> superContext) {
		super(superContext);
		// RunEnvironment.getInstance().getCurrentSchedule().schedule(this);
		this.name = id++;
		logger.info("AreaContext initialised for super context "
				+ superContext.getId());
	}

	// there is a bug in FeatureAgent2 - the condition must be >= 0 rather than
	// >0 (that's why the first method must be non-getX)
	public int test() {
		return 10;
	}
	
	// @ScheduledMethod(start = 1, interval = 1, priority =
	// ScheduleParameters.FIRST_PRIORITY)
	public void checkConvergence() {
		int curTick = (int) RunEnvironment.getInstance().getCurrentSchedule()
		.getTickCount();
		// extend simulation if required:
		int endTick = ((Integer) PmParameterManager.getParameter(BasicPa.END_TICK)).intValue();
		double x = getAvgAbsOpin();

		// % entTick is too late...
		if (curTick > ContextCreator.X_INTERVAL && 
				RunEnvironment.getInstance().getCurrentSchedule().getTickCount()
				% endTick == 0) {

			double xLast = xLastMap.get(curTick - ContextCreator.X_INTERVAL).doubleValue();

			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug(this + "> Average Opinion: " + x + " / Last Average opinion: " + xLast);
			}
			// LOGGING ->
	
			if (Math.abs(xLast - x) > ContextCreator.X_CHANGED_THRESHOLD) { // && (x < X_MAX))
																// {
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Extend end tick to " + (curTick + endTick));
				} 
				// LOGGING ->
			}
			else {
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Stop simulation!");
				}
				// LOGGING ->
				convergenceTick = curTick;
			}
		}
		xLastMap.put(curTick, x);
		xLastMap.remove(curTick - ContextCreator.X_INTERVAL);		
	}
	
	public double getConverTick() {
		return this.convergenceTick;
	}

	public double getAvgOpin() {
		double sum = 0.0;
		int counter = 0;
		for (Object o : this) {
			if (o instanceof SonomodeAgent) {
				sum += ((SonomodeAgent) o).getOpinion();
				counter++;
			}
		}
		return sum / counter;
	}

	public double getAvgAbsOpin() {
		double sum = 0.0;
		int counter = 0;
		for (Object o : this) {
			if (o instanceof SonomodeAgent) {
				sum += Math.abs(((SonomodeAgent) o).getOpinion());
				counter++;
			}
		}
		return sum / counter;
	}

	public double getAvgUncert() {
		double sum = 0.0;
		int counter = 0;
		for (Object o : this) {
			if (o instanceof SonomodeAgent) {
				sum += ((SonomodeAgent) o).getUncertaint();
				counter++;

				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Added for " + o + ": "
							+ ((SonomodeAgent) o).getUncertaint() + "( sum: "
							+ sum + " / counter: " + counter + ")");
				}
				// LOGGING ->

			}
		}
		return sum / counter;
	}

	public double getNumClusters() {
		double epsilon = ((Double) PmParameterManager
				.getParameter(BasicPa.EPSILON)).doubleValue();

		List<SonomodeAgent> orderedModerateAgents = new ArrayList<SonomodeAgent>();

		for (SonomodeAgent agent : this.getAgentLayer(SonomodeAgent.class)) {
			if (agent.isModerate()) {
				orderedModerateAgents.add(agent);
			}
		}

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Number of ordered moderates: "
					+ orderedModerateAgents.size());
		}
		if (orderedModerateAgents.size() == 0) {
			logger.warn("No moderates...");
		}
		// LOGGING ->

		Collections.sort(orderedModerateAgents,
				new Comparator<SonomodeAgent>() {
					@Override
					public int compare(SonomodeAgent o1, SonomodeAgent o2) {
						return Double.compare(o1.getOpinion(), o2.getOpinion());
					}
				});

		List<Integer> clusters = new ArrayList<Integer>();

		double denominator = 0.0;
		if (orderedModerateAgents.size() > 0) {
			SonomodeAgent first = orderedModerateAgents.get(0);
			// for the first loop, agent = first holds
			int clusterMemberCounter = 0;
			for (SonomodeAgent agent : orderedModerateAgents) {
				if (agent.getOpinion() - first.getOpinion() < epsilon) {
					clusterMemberCounter++;
				} else {
					clusters.add(new Integer(clusterMemberCounter));
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug("Denominator: "
								+ denominator
								+ " || p: "
								+ Math.pow(
										(double) clusterMemberCounter
												/ (double) orderedModerateAgents
														.size(), 2.0));
					}
					// LOGGING ->

					denominator += Math.pow((double) clusterMemberCounter
							/ (double) orderedModerateAgents.size(), 2.0);
					clusterMemberCounter = 1;
				}
				first = agent;
			}
			clusters.add(new Integer(clusterMemberCounter));

			denominator += Math.pow((double) clusterMemberCounter
					/ (double) orderedModerateAgents.size(), 2.0);
		} else {
			denominator = 1.0;
		}

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			int counter = 0;
			for (Integer i : clusters) {
				counter += i.intValue();
			}
			logger.debug("Sum of cluster members needs to be equal to number of moderates: "
					+ counter + " = " + orderedModerateAgents.size() + "!");
			logger.debug("n: " + (1.0 / denominator));
		}
		// LOGGING ->

		return 1.0 / denominator;
	}

	@Override
	public String toString() {
		return "AreaContext-" + name;
	}
}
