/**
 * 
 */
package de.cesr.sonomode.core;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import de.cesr.parma.core.PmParameterManager;
import de.cesr.sesamgim.init.GimModelInitialiser;
import de.cesr.sesamgim.init.agent.GimAgentInitialiseService;
import de.cesr.sesamgim.init.agent.GimAgentInitialiser;
import de.cesr.sesamgim.param.GInitialisersPa;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.param.MilieuPa;
import de.cesr.sonomode.util.MilieuPrefParameterMap;

/**
 * @author Sascha Holzhauer
 *
 */
public class AgentMilieuInitialiser implements GimAgentInitialiser<SonomodeAgent> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(AgentMilieuInitialiser.class);

	@Override
	public List<SonomodeAgent> initAgents(
			GimModelInitialiser<SonomodeAgent> ginit) {
		
		GimAgentInitialiseService<SonomodeAgent> gAgentIS = null;
		try {
			gAgentIS = ((Class<GimAgentInitialiseService<SonomodeAgent>>)
					PmParameterManager.getParameter(GInitialisersPa.IS_AGENTS)).newInstance();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		
		ArrayList<SonomodeAgent> agents = new ArrayList<SonomodeAgent>();
		
		int num_total = ((Integer)PmParameterManager.getParameter(BasicPa.NUM_AGENTS)).intValue();
		int num_extremists = (int) (num_total * ((Double)PmParameterManager.getParameter(BasicPa.PROP_EXTREMISTS)).doubleValue());
		
		// enforce equal shares of each extreme opinion.
		if (num_extremists % 2 != 0) {
			num_extremists++;
		}
		
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Number of extremists: " + num_extremists + "; total number of agents: " + num_total);
		}
		// LOGGING ->
		
		if (((MilieuPrefParameterMap) PmParameterManager
				.getParameter(MilieuPa.MILIEU_PREF_PARAMS)).size() == 3) {
			for (int i = 0; i < num_extremists / 2; i++) {
				agents.add(gAgentIS.initAgent(1));
				agents.add(gAgentIS.initAgent(3));
			}
		} else {
			for (int i = 0; i < num_extremists; i++) {
				agents.add(gAgentIS.initAgent(1));
			}
		}
		

		for (int i = 0; i < num_total - num_extremists; i++) {
			agents.add(gAgentIS.initAgent(2));
		}
		return agents;
	}

}
