/**
 * This file is part of
 * 
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant
 *
 * Copyright (C) 2011 Center for Environmental Systems Research, Kassel, Germany
 * 
 * SesamGIM - Geographical Initialisation for Milieu Agents is free software:
 * You can redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 *  
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Sascha Holzhauer
 * Center for Environmental Systems Research, Kassel
 */
package de.cesr.sonomode.core;


import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;

import org.apache.commons.collections15.BidiMap;
import org.apache.commons.collections15.bidimap.DualHashBidiMap;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import repast.simphony.context.Context;
import repast.simphony.dataLoader.ContextBuilder;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.gis.Geography;
import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;
import de.cesr.more.basic.MManager;
import de.cesr.more.basic.MNetworkManager;
import de.cesr.more.basic.edge.MoreEdge;
import de.cesr.more.basic.network.MDirectedNetwork;
import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.building.edge.MDefaultEdgeFactory;
import de.cesr.more.building.edge.MoreEdgeFactory;
import de.cesr.more.manipulate.agent.MEgoNetworkManagerComp;
import de.cesr.more.manipulate.agent.MoreEgoNetworkEvent;
import de.cesr.more.manipulate.agent.MoreEgoNetworkManagerComp;
import de.cesr.more.manipulate.agent.MoreEgoNetworkProcessor;
import de.cesr.more.manipulate.agent.events.MLinkManagingEvent;
import de.cesr.more.manipulate.agent.events.MLinkWeightUpdatingEvent;
import de.cesr.more.manipulate.edge.MDefaultNetworkEdgeModifier;
import de.cesr.more.manipulate.edge.MoreNetworkEdgeModifier;
import de.cesr.more.measures.node.MVertexSimilarityMeasureCalculator;
import de.cesr.more.param.MBasicPa;
import de.cesr.more.param.MNetBuildBhPa;
import de.cesr.more.param.MNetManipulatePa;
import de.cesr.more.param.MNetworkBuildingPa;
import de.cesr.more.param.MRandomPa;
import de.cesr.more.param.MSqlPa;
import de.cesr.more.param.reader.MMilieuNetDataReader;
import de.cesr.more.rs.building.MGeoRsLattice2DNetworkBuilder;
import de.cesr.more.rs.building.MGeoRsNetworkService;
import de.cesr.more.rs.building.MoreRsNetworkBuilder;
import de.cesr.more.rs.building.edge.MGeoRsNetworkEdgeModifier;
import de.cesr.more.rs.building.edge.MRsEdgeFactory;
import de.cesr.more.rs.geo.util.MGeographyWrapper;
import de.cesr.more.rs.network.MoreRsNetwork;
import de.cesr.more.rs.util.MRsSchedule;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.parma.definition.PmFrameworkPa;
import de.cesr.parma.reader.PmDbParameterReader;
import de.cesr.parma.reader.PmDbXmlParameterReader;
import de.cesr.parma.reader.PmXmlParameterReader;
import de.cesr.sesamgim.aggregate.GAggregateAgents;
import de.cesr.sesamgim.aggregate.GGeographyDissimilarityProvider;
import de.cesr.sesamgim.aggregate.GimAgentDissimilarityProvider;
import de.cesr.sesamgim.context.GimAreaContext;
import de.cesr.sesamgim.core.GManager;
import de.cesr.sesamgim.init.GDefaultModelInitialiser;
import de.cesr.sesamgim.init.GimModelInitialiser;
import de.cesr.sesamgim.init.agent.GDbAgentInitialiser;
import de.cesr.sesamgim.init.agent.GimAgentInitialiser;
import de.cesr.sesamgim.param.GAggregatePa;
import de.cesr.sesamgim.param.GBasicPa;
import de.cesr.sesamgim.param.GInitialisersPa;
import de.cesr.sesamgim.param.GMilieuPa;
import de.cesr.sesamgim.param.GRandomPa;
import de.cesr.sesamgim.param.GSqlPa;
import de.cesr.sesamgim.util.GShapefileService;
import de.cesr.sesamgim.util.GTorusCoordinate;
import de.cesr.sonomode.agent.AbstractSonomodeAgent;
import de.cesr.sonomode.agent.ModerateSonomodeAgent;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.analysis.AgentDissimilarityProvider;
import de.cesr.sonomode.analysis.AgentPropertyAggregator;
import de.cesr.sonomode.analysis.SocialNetworkAnalysis;
import de.cesr.sonomode.context.AreaContext;
import de.cesr.sonomode.context.AreaContext.AreaContextIS;
import de.cesr.sonomode.context.SocialContext;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.param.MilieuPa;
import de.cesr.sonomode.param.RandomPa;
import de.cesr.sonomode.param.SocNetPa;
import de.cesr.sonomode.param.SqlPa;
import de.cesr.sonomode.param.reader.MilieuPrefDataReader;
import de.cesr.sonomode.socnet.SonomodeEdge;
import de.cesr.sonomode.socnet.builder.NetworkUtils;
import de.cesr.sonomode.socnet.builder.SonomodeEdgeFactory;
import de.cesr.sonomode.util.DataProvider;
import de.cesr.sonomode.util.MilieuPrefParameterMap;
import de.cesr.sonomode.util.Parameters;
import de.cesr.sonomode.util.VersionInfo;
import de.cesr.uranus.core.URandomService;

public class ContextCreator implements ContextBuilder<Object> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(ContextCreator.class);
	
	static private Logger basicInfologger = Logger.getLogger(ContextCreator.class.getName() + ".basic");
	
	static long startTimeMillis;
	
	static final double NUM_AGENT_PRECISION = 0.001;
	
	static final public double X_CHANGED_THRESHOLD = 0.01;
	
	static final public int X_INTERVAL = 200;

	static final double X_MAX = 0.9;
	
	private final Map<Integer, Double> xLastMap = new HashMap<Integer, Double>();
	
	private Parameters 			parameters = null;
	
	private MoreRsNetworkBuilder<SonomodeAgent, SonomodeEdge>		networkBuilder;

	
	@Override
	public Context<Object> build(Context<Object> context) {
		
		initTimingInformation();
		
		// crate a new sub context for social network only:
		Context<SonomodeAgent> socialContext = new SocialContext(ModelManager.SOC_CONTEXT_NAME);
		context.addSubContext(socialContext);
		ModelManager.getMan().setSocialContext(socialContext);
		
		this.initDbSettingsParameters();

		this.assignRsParameters();
		
		this.initParameters(context);
		
		this.initRandomDistributions();
		
		this.initSesamGim(context);
		
		try {
			MRsEdgeFactory<SonomodeAgent, SonomodeEdge> eFac = SonomodeEdgeFactory
					.getInstance();
			
			this.networkBuilder = (MoreRsNetworkBuilder<SonomodeAgent, SonomodeEdge>) ((Class<?>) PmParameterManager.getParameter(SocNetPa.NETWORK_BUILDER)).
			getConstructor(MoreEdgeFactory.class, String.class).newInstance(eFac,
					PmParameterManager.getParameter(SocNetPa.SOCNET_NAME));
		
			this.networkBuilder.setContext(socialContext);
			if (networkBuilder instanceof MGeoRsNetworkService) {
				if (((Integer) PmParameterManager
							.getParameter(GBasicPa.INIT_PARAM_ID)).intValue() != 0) {
					((MGeoRsNetworkService)this.networkBuilder).setGeography(GManager.getMan().getRootGeography());
					if ((Boolean)PmParameterManager.getParameter(BasicPa.USE_DB_AGENT_INITIALISER)) {
						PmParameterManager.setParameter(GInitialisersPa.IS_AGENTS,
								DbAgentInitService.class);
					}
				} else {
					logger.warn("Geo-network-bilder requires geography != null, but GBasicPa.INIT_PARAM_ID == 0!");
					((MGeoRsNetworkService) this.networkBuilder)
							.setGeography(null);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		List<SonomodeAgent> agents = this.initAgents(context);

		// <- LOGGING
		logger.info("Init social network...");
		// LOGGING ->

		this.initSocialNetwork(agents);
		
		this.initSocialNetworkAnalysis();
		
		// set end tick
		// RunEnvironment.getInstance().endAt(((Integer) PmParameterManager.getParameter(BasicPa.END_TICK)).intValue());
		RunEnvironment.getInstance().getCurrentSchedule().schedule(this);
		
		for (int i = 0; i < (Integer) PmParameterManager
				.getParameter(BasicPa.AGGREGATE_AGENT_AMOUNT); i++) {
			Collection<SonomodeAgent> agentCol = new HashSet<SonomodeAgent>();
			for (SonomodeAgent agent : socialContext) {
				agentCol.add(agent);
			}
			this.aggregateAgents(
					context,
					(MoreRsNetwork<SonomodeAgent, SonomodeEdge>) MNetworkManager
							.getNetwork(ModelManager.SOCNET_NAME), agentCol);
		}

		// schedule output of shapefiles
		if (((Double)PmParameterManager.getParameter(BasicPa.WRITE_SHAPEFILES)).doubleValue() > 0.0) {
			RunEnvironment.getInstance().getCurrentSchedule().schedule(ScheduleParameters.createOneTime(
									((Double) PmParameterManager
											.getParameter(BasicPa.WRITE_SHAPEFILES))
											.doubleValue(), 100000), this,
							"writeShapes");
		} else if(((Double)PmParameterManager.getParameter(BasicPa.WRITE_SHAPEFILES)).doubleValue() < 0) {
			RunEnvironment.getInstance().getCurrentSchedule().schedule(ScheduleParameters.createRepeating(0,
									(-1)
											* ((Double) PmParameterManager
													.getParameter(BasicPa.WRITE_SHAPEFILES))
													.doubleValue(), 100000),
							this, "writeShapes");
		}
		
		//schedule output of networks
		int outputNetwork = ((Integer)PmParameterManager.getParameter(SocNetPa.OUTPUT_NETWORK)).intValue();
		if (outputNetwork > 0) {
			RunEnvironment.getInstance().getCurrentSchedule().schedule(ScheduleParameters.createOneTime(
					outputNetwork , -Double.MAX_VALUE), this, "writeNetwork");
		} else if(outputNetwork < 0) {
			RunEnvironment.getInstance().getCurrentSchedule().schedule(ScheduleParameters.createRepeating(0,
					(-1)*outputNetwork, -Double.MAX_VALUE),this,"writeNetwork");
		}
				
		return context;
	}


	/**
	 * 
	 */
	public void writeShapes() {
		if (((Integer) PmParameterManager.getParameter(GBasicPa.INIT_PARAM_ID))
				.intValue() > 0) {
			logger.info("Write shapefiles...");
			List<String> features = Arrays.asList(
					"de.cesr.sonomode.agent.ExtremeSonomodeAgent",
					"de.cesr.sonomode.agent.ModerateSonomodeAgent",
					"de.cesr.sonomode.context.AreaContext",
					"de.cesr.sonomode.socnet.SonomodeEdge");

			GShapefileService.writeShapeFile(GManager.getMan()
					.getUtmRootGeography(),
					(ModelManager.getMan().getRunId() > 0 ? ModelManager
							.getMan().getRunId() : "Unknown")
							+ "/"
							+ RunEnvironment.getInstance().getCurrentSchedule()
									.getTickCount() + "/", features);
		}
	}
	
	
	@SuppressWarnings("unchecked")
	public void writeNetwork() {
		logger.info("Write network...");
		File file = new File(((String) PmParameterManager.getParameter(SocNetPa.NETWORK_OUTPUT_FOLDER) + "/" + 
				(ModelManager.getMan().getRunId() > 0 ? ModelManager.getMan().getRunId() : "Unknown") + "/"
				+ RunEnvironment.getInstance().getCurrentSchedule().getTickCount() + "/" + ModelManager.SOCNET_NAME) + ".graphml");

		NetworkUtils.outputNetwork((MoreNetwork<SonomodeAgent, SonomodeEdge>) MNetworkManager.getNetwork(ModelManager.SOCNET_NAME), file);
	}
	
	/**
	 * Apparently, RS sets its random seed after Context Building
	 */
	@ScheduledMethod(start = 0, interval = 0)
	public void setRsRandomSeed() {
		RandomHelper.setSeed(((Integer) PmParameterManager.getParameter(RandomPa.RANDOM_SEED)).intValue());
	}
	
	protected boolean areasConverged() {
		boolean converged = true;
		for (Object o : RunState.getInstance().getMasterContext()
				.getSubContexts()) {
			if (o instanceof AreaContext) {
				if (Double.isNaN(((AreaContext) o).getConverTick())) {
					converged = false;

					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug("Not converged: " + o);
					}
					// LOGGING ->
				} else {
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug("Converged: " + o + "("
								+ ((AreaContext) o).getConverTick() + ")");
					}
					// LOGGING ->
				}
			}
		}
		return converged;
	}

	/**
	 * Anchor method for Repast Simphony to schedule the simulation.
	 */
	@ScheduledMethod(start = 1, interval = 1, priority = Double.POSITIVE_INFINITY)
	public void stepIt() {
		logger.info(">>>>> Simulating timestep " + RunEnvironment.getInstance().getCurrentSchedule().getTickCount() + " <<<<<<");
		
		for (Object o : RunState.getInstance().getMasterContext()
				.getSubContexts()) {
			if (o instanceof AreaContext) {
				((AreaContext) o).checkConvergence();
			}
		}

		int curTick = (int) RunEnvironment.getInstance().getCurrentSchedule()
				.getTickCount();
		// extend simulation if required:
		int endTick = ((Integer) PmParameterManager.getParameter(BasicPa.END_TICK)).intValue();
		double x = ((SocialContext) ModelManager.getMan().getSocialContext()).getAvgOpinion();
		
		// % entTick is too late...
		if (curTick > X_INTERVAL
				&& RunEnvironment.getInstance().getCurrentSchedule()
						.getTickCount()
						% endTick == 0) {

			double xLast = xLastMap.get(curTick - X_INTERVAL).doubleValue();

			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("Average Opinion: " + x + " / Last Average opinion: " + xLast);
			}
			// LOGGING ->
			
			if (!areasConverged() || Math.abs(xLast - x) > X_CHANGED_THRESHOLD) { // &&
																					// (x
																					// <
																					// X_MAX))
																// {
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Extend end tick to " + (curTick + endTick));
				} 
				// LOGGING ->
			}
			else {
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Stop simulation!");
				}
				// LOGGING ->
				RunEnvironment.getInstance().endRun();
			}
		}
		xLastMap.put(curTick, x);
		xLastMap.remove(curTick - X_INTERVAL);
	}
	

	private void initTimingInformation() {
		startTimeMillis = System.currentTimeMillis();
		basicInfologger.info("Started at " + new SimpleDateFormat("HH:mm:ss:SS").format(new Date(startTimeMillis)));
		// Required to reset log4j configuration in order to use the newly set system property
		SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd-HH-mm-ss");
		System.setProperty("current.time", dateFormat.format(new Date()));
		LogManager.resetConfiguration();
		ClassLoader cl = this.getClass().getClassLoader();
		if (System.getProperty("log4j.configuration") != null) {
			if (cl.getResource(System.getProperty("log4j.configuration")) != null) {
				PropertyConfigurator.configure(cl.getResource(System.getProperty("log4j.configuration")));
			} else {
				System.err.println("The specified log4j configuration file (" +
						System.getProperty("log4j.configuration") + ") is not at the classpath!");
			}
		}
		else {
			if (cl.getResource("log4j.properties") != null) {
				PropertyConfigurator.configure(cl.getResource("log4j.properties"));
			} else {
				System.err.println("There is no default log4j configuration file (log4j.properties) at the classpath!");
			}
		}
	}

	private void initParameters(Context<Object> rootContext) {
		 
		// SesamGIM
		PmParameterManager.setParameter(GInitialisersPa.INITIALISER_MARKETCELL,
				null);
		PmParameterManager
				.setParameter(GInitialisersPa.INITIALISER_GROUP, null);
		PmParameterManager.copyParameterValue(BasicPa.AREAS_SHAPEFILE,
				GBasicPa.AREAS_SHAPEFILE);
		PmParameterManager.setParameter(GBasicPa.FIELD_UPPER_X, new Double(10000));
		PmParameterManager.setParameter(GBasicPa.FIELD_UPPER_Y, new Double(10000));
		PmParameterManager.setParameter(GBasicPa.CRS, "EPSG:32632"); // WGS 84 / UTM zone 32N)
		PmParameterManager.setParameter(GBasicPa.COORDINATE_CLASS, GTorusCoordinate.class);
		PmParameterManager.setParameter(GBasicPa.SHAPEFILE_OUTPUT_FOLDER, PmParameterManager.getParameter(BasicPa.SHAPEFILE_OUTPUT_FOLDER));
		PmParameterManager.setParameter(GInitialisersPa.IS_AREA,
				AreaContextIS.class);
		PmParameterManager.setParameter(GSqlPa.TBLNAME_INIT_AGENTS,
				PmParameterManager.getParameter(SqlPa.TBLNAME_INIT_AGENTS));
		PmParameterManager.copyParameterValue(BasicPa.PARAM_SET_ID,
				GAggregatePa.AGGREGATE_PARAM_ID);
		PmParameterManager.setParameter(GAggregatePa.AGENT_PROPERTY_AGGREGATOR,
				new AgentPropertyAggregator());

		// init parameter from table
		PmParameterManager.setParameter(PmFrameworkPa.PARAM_SET_ID, PmParameterManager.getParameter(BasicPa.PARAM_SET_ID));
		
		PmParameterManager.registerReader(new PmDbParameterReader(BasicPa.PARAM_SET_ID));
		PmParameterManager.registerReader(new MilieuPrefDataReader());
		PmParameterManager.registerReader(new PmXmlParameterReader());
		
		
		PmParameterManager.init();
		
		// milieu number related stuff:
		BidiMap<String, Integer> milieuKeysMap = new DualHashBidiMap<String, Integer>();
		if (((MilieuPrefParameterMap) PmParameterManager
				.getParameter(MilieuPa.MILIEU_PREF_PARAMS)).size() > 2) {
			PmParameterManager.setParameter(GInitialisersPa.IS_AGENTS,
					ThreeMilieuAgentInitService.class);
			milieuKeysMap.put("EXT+", new Integer(1));
			milieuKeysMap.put("MOD", new Integer(2));
			milieuKeysMap.put("EXT-", new Integer(3));
		} else {
			PmParameterManager.setParameter(GInitialisersPa.IS_AGENTS,
					AgentInitService.class);
			milieuKeysMap.put("EXT", new Integer(1));
			milieuKeysMap.put("MOD", new Integer(2));
		}
		PmParameterManager.setParameter(MNetworkBuildingPa.MILIEUS,
				milieuKeysMap);

		PmParameterManager.logParameterValues(MNetworkBuildingPa.values());

		// required to read milieu preferece table parameters:
		PmParameterManager.copyParameterValue(SqlPa.TBLNAME_NETPREF, MSqlPa.TBLNAME_NET_PREFS);
		PmParameterManager.copyParameterValue(SqlPa.TBLNAME_NETPREF_LINKS, MSqlPa.TBLNAME_NET_PREFS_LINKS);
		PmParameterManager.copyParameterValue(MilieuPa.MILIEU_NETPREFS_PARAM, MNetworkBuildingPa.MILIEU_NETPREFS_PARAMID);
		new MMilieuNetDataReader().initParameters();
		
		PmParameterManager.logParameterValues(MNetworkBuildingPa.values());
		
		// assign Sonomode params to MoRe:
		PmParameterManager.setParameter(MRandomPa.RANDOM_SEED_NETWORK_BUILDING,
				((Integer)PmParameterManager.getParameter(RandomPa.RANDOM_SEED_NETWORKINIT)).intValue());

		// Social net parameter
		PmParameterManager.setParameter(PmFrameworkPa.DB_SETTINGS_FILE, 
				"./config/DBSettingsSocNet.xml");
		new PmDbXmlParameterReader().initParameters();
		new PmDbParameterReader(SocNetPa.SOCNET_PARAM_ID).initParameters();
		
		// read XML parameters a second time after reading SocNet-DB (first
		// reading of XML required for SocNet DB):
		new PmXmlParameterReader().initParameters();

		GManager.getMan().setRootContext(rootContext);
		PmParameterManager.setParameter(MBasicPa.ROOT_GEOGRAPHY, GManager.getMan().getRootGeography());
		PmParameterManager.setParameter(
				MNetworkBuildingPa.BUILD_WSSM_CONSIDER_SOURCES, Boolean.FALSE);
		
		PmParameterManager.logParameterValues(BasicPa.values(),
				MilieuPa.values(), SocNetPa.values(), RandomPa.values(),
				SqlPa.values(), MRandomPa.values(), MBasicPa.values(),
				MNetworkBuildingPa.values(), MNetManipulatePa.values(),
				MNetBuildBhPa.values(),
				MSqlPa.values(), PmFrameworkPa.values(), GBasicPa.values(),
				GRandomPa.values(), GSqlPa.values(), GMilieuPa.values());

		// check if num agents is valid
		if (((Class<?>)PmParameterManager.getParameter(SocNetPa.NETWORK_BUILDER)) == MGeoRsLattice2DNetworkBuilder.class) {
			if (Math.sqrt(((Integer)PmParameterManager.getParameter(BasicPa.NUM_AGENTS)).intValue()) % 1.0 > NUM_AGENT_PRECISION) {
				logger.error("When Lattic2DNetworkBuilder is used, the number of agents need to have an integer square root.");
			}
		}

		DataProvider.disconnect();
	}


	private void initDbSettingsParameters() {
		PmParameterManager.setParameter(PmFrameworkPa.PARAM_SET_ID, 
				PmParameterManager.getParameter(BasicPa.PARAM_SET_ID));
		PmParameterManager.setParameter(PmFrameworkPa.XML_PARAMETER_FILE, PmParameterManager.getParameter(BasicPa.XML_PARAMETER_FILE));
		// set db xml parameter file before the ModelManager.getRunID() is called in assignRsParameters()
		PmParameterManager.copyParameterValue(SqlPa.DB_SETTINGS_FILE, PmFrameworkPa.DB_SETTINGS_FILE);
		new PmDbXmlParameterReader().initParameters();
	}	
	
	private void initRandomDistributions() {
		RandomHelper.registerDistribution(ModelManager.RANDOM_OPINION, URandomService.getURandomService().getUniform(-1.0, 1.0, new MersenneTwister(((Integer) PmParameterManager.getParameter(
				RandomPa.RANDOM_SEED_OPINION)).intValue())));
		RandomHelper.registerDistribution(ModelManager.RANDOM_NOISE, URandomService.getURandomService().getUniform(0.0, 1.0, new MersenneTwister(((Integer) PmParameterManager.getParameter(
				RandomPa.RANDOM_SEED_NOISE)).intValue())));
		
		RandomHelper.registerDistribution(ModelManager.RANDOM_NOISE_AMOUNT, URandomService.getURandomService().getUniform(
				-1 * ((Double)PmParameterManager.getParameter(RandomPa.NOISE_AMOUNT)).doubleValue(),
				((Double)PmParameterManager.getParameter(RandomPa.NOISE_AMOUNT)).doubleValue(),
				new MersenneTwister(((Integer) PmParameterManager.getParameter(
				RandomPa.RANDOM_SEED_NOISE_AMOUNT)).intValue())));
		
		MManager.getURandomService().setSeed(((Integer) PmParameterManager.getParameter(RandomPa.RANDOM_SEED_NETWORKINIT)).intValue());

		MManager.getURandomService()
				.registerGenerator(
						(String) PmParameterManager
								.getParameter(MRandomPa.RND_STREAM_NETWORK_BUILDING),
						new MersenneTwister(
								((Integer) PmParameterManager
										.getParameter(MRandomPa.RANDOM_SEED_NETWORK_BUILDING))
										.intValue()));

		MManager.getURandomService()
				.registerDistribution(
						new Uniform(
								MManager.getURandomService()
										.getGenerator(
												(String) PmParameterManager
														.getParameter(MRandomPa.RND_STREAM_NETWORK_BUILDING))),
						(String) PmParameterManager
								.getParameter(MRandomPa.RND_UNIFORM_DIST_NETWORK_BUILDING));
	}
	
	private void assignRsParameters() {
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Assign RS parameters");
		}
		// LOGGING ->
		// Set PARAM_ID and VERSION as Repast Simphony parameters
		if (RunEnvironment.getInstance().getParameters().getSchema().contains("PARAM_SET_ID")) {
			PmParameterManager.setParameter(BasicPa.PARAM_SET_ID, ((Integer) RunEnvironment.getInstance().getParameters()
					.getValue("PARAM_SET_ID")).intValue());
		} else {
			// WORK around (it is not possible to add parameters to the RS default implementation of Parameters)
			// if parameters is already in use, the parameter needs to be added there
			
			// Note: Only default values for PARAM_SET_ID may be considered since other readers are not
			// initialised before.
			copyRsParameters(); // checks if parameters == null!
			parameters.addParameter("PARAM_SET_ID", Integer.class, 
					PmParameterManager.getParameter(BasicPa.PARAM_SET_ID), false);
		}

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Batch number: " + RunState.getInstance().getRunInfo().getRunNumber());
		}
		// LOGGING ->

		if (RunEnvironment.getInstance().getParameters().getSchema().contains("RUN_ID") && (parameters == null)) {
			//parameter == null means that no custom parameter object is required yet
			if (RunState.getInstance().getRunInfo().getRunNumber() == 1) {
				// in first batch run, get run number from RS if it is defined there
				ModelManager.setRunId(((Integer) RunEnvironment.getInstance().getParameters().getValue("RUN_ID"))
						.intValue());
			} else {
				// then, the ModelManager via DB-Outputter takes care of the run ID:
				RunEnvironment.getInstance().getParameters().setValue("RUN_ID", ModelManager.getMan().getRunId());
			}
		} else {
			// WORK around (it is not possible to add parameters to the RS default implementation of Parameters)
			// if parameters is already in use, the parameter needs to be added there
			copyRsParameters(); // checks if parameters == null!
			parameters.addParameter("RUN_ID", Integer.class, ModelManager.getMan().getRunId(), false);
		}
		
		// <- LOGGING
		basicInfologger.info("RunID: " + ModelManager.getMan().getRunId() + " | Param Set ID: " + 
				PmParameterManager.getParameter(BasicPa.PARAM_SET_ID));
		// LOGGING ->
		
		String versionInfo = "Sonomode: " + VersionInfo.revisionNumber + "/" + VersionInfo.timeStamp;
		
		if (RunEnvironment.getInstance().getParameters().getSchema().contains("VERSION") && (parameters == null)) {
			RunEnvironment.getInstance().getParameters().setValue(
					"VERSION", versionInfo);
		} else {
			// WORK around (it is not possible to add parameters to the RS default implementation of Parameters):
			// if parameters is already in use, the parameter needs to be added there
			copyRsParameters(); // checks if parameters == null!
			parameters.addParameter("VERSION", String.class, versionInfo, false);
		}
	}
	
	/**
	 * 
	 */
	private void copyRsParameters() {
		if (parameters == null) {
			parameters = new Parameters();
			// copy parameter from old parameters to new parameters:
			for (String name : RunEnvironment.getInstance().getParameters().getSchema().parameterNames()) {
				parameters.addParameter(name, Object.class,
						RunEnvironment.getInstance().getParameters().getValue(name), 
						RunEnvironment.getInstance().getParameters().isReadOnly(name));
			}
			RunEnvironment.getInstance().setParameters(parameters);
		}
	}
	
	private void initSesamGim(Context<Object> rootContext) {
		GManager.getMan().setRootContext(rootContext);
	}
	
	private List<SonomodeAgent> initAgents(Context<Object> context) {
		
		// <- LOGGING
		logger.info("Init agents...");
		// LOGGING ->
		
		GimAgentInitialiser<SonomodeAgent> agentInitialiser;
		if (((Integer) PmParameterManager.getParameter(GBasicPa.INIT_PARAM_ID)).intValue() > 0) {
			agentInitialiser = new GDbAgentInitialiser<SonomodeAgent>();
		} else {
			agentInitialiser = new AgentMilieuInitialiser();
		}
		
		GimModelInitialiser<SonomodeAgent> gInitialiser = new GDefaultModelInitialiser<SonomodeAgent>();

		gInitialiser.initAreas();

		List<SonomodeAgent> agents = new ArrayList<SonomodeAgent>(agentInitialiser.initAgents(gInitialiser));
		
		if (agents.size() != ((Integer) PmParameterManager
				.getParameter(BasicPa.NUM_AGENTS)).intValue()) {
			logger.warn("Number of agents does not match BasicPa.NUM_AGENTS!");
		}
		
		// re-initialise agents' opinion with respect to area:
		if ((Integer) PmParameterManager
				.getParameter(BasicPa.SPATIAL_INITIAL_OPINION_ID) > 0) {
			MGeographyWrapper<Object> geoWrapper = new MGeographyWrapper<Object>(
					GManager.getMan().getRootGeography());
			Uniform uniform = (Uniform) RandomHelper
					.getDistribution(ModelManager.RANDOM_OPINION);
			for (SonomodeAgent agent : agents) {
				if (agent instanceof ModerateSonomodeAgent) {
					// identify agent's area
					int areaID = geoWrapper.getContainingAreaContext(agent,
							GimAreaContext.class).getAreaId();

					// assign new initial opinion
					double o = Double.NaN;
					double random = uniform.nextDouble();
					switch ((Integer) PmParameterManager
							.getParameter(BasicPa.SPATIAL_INITIAL_OPINION_ID)) {
					case 1:
						o = -1 + (areaID - 1) * 2.0 / 9.0 + Math.abs(random)
								/ 9.0;
						break;
					case 2:
						o = (areaID - 1) / 9.0 * Math.signum(random)
								+ random / 9.0;
						break;
					default:
						o = random;
					}

					agent.setOpinion(o);
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug("AreaID: "
								+ areaID
								+ "("
								+ geoWrapper.getContainingAreaContext(agent,
										GimAreaContext.class).toString()
								+ ") | Opinion: " + o);
					}
					// LOGGING ->
				}
			}
		}

		// shuffle agents (required to mix extremists with moderates!):
		if (((Integer)PmParameterManager.getParameter(SocNetPa.SHUFFLE_AGENTS_BEFORE_NET_BUILT)).intValue() > 0) {
			Collections.shuffle(agents, new Random(((Integer)PmParameterManager.getParameter(RandomPa.RANDOM_SEED)).intValue()));
			logger.debug("Shuffle order: " + agents);
		}
		
		ModelManager.getMan().getSocialContext().addAll(agents);
		
		// configure link management
		Map<MoreEgoNetworkEvent, MoreEgoNetworkProcessor<SonomodeAgent, SonomodeEdge>> map = MEgoNetworkManagerComp
				.getEmptyProcessorMap();
		try {
			
			MoreNetworkEdgeModifier<SonomodeAgent, SonomodeEdge> edgeMod;
			if (((Integer)PmParameterManager.getParameter(GBasicPa.INIT_PARAM_ID)).intValue() > 0) {
				edgeMod = new MGeoRsNetworkEdgeModifier<SonomodeAgent, SonomodeEdge>(
						SonomodeEdgeFactory.getInstance());
			} else {
				edgeMod = new MDefaultNetworkEdgeModifier<SonomodeAgent, SonomodeEdge>(SonomodeEdgeFactory.getInstance());
			}

			map.put(MLinkManagingEvent.getInstance(),
					((Class<MoreEgoNetworkProcessor<SonomodeAgent, SonomodeEdge>>) PmParameterManager
							.getParameter(SocNetPa.DYN_EDGE_MANAGER))
							.getConstructor(MoreNetworkEdgeModifier.class,
									Geography.class).newInstance(edgeMod,
									GManager.getMan().getRootGeography()));
			map.put(MLinkWeightUpdatingEvent.getInstance(),
					((Class<MoreEgoNetworkProcessor<SonomodeAgent, SonomodeEdge>>) PmParameterManager
							.getParameter(SocNetPa.DYN_EDGE_WEIGHT_UPDATER))
							.newInstance());
		} catch (Exception e) {
			e.printStackTrace();
		}
		MoreEgoNetworkManagerComp<SonomodeAgent, SonomodeEdge> egoNetManager = new MEgoNetworkManagerComp<SonomodeAgent, SonomodeEdge>(
				map);
		for (SonomodeAgent agent : agents) {
			agent.setEgoNetworkManager(egoNetManager);
			agent.initDynamicNetworkCode();
		}
		return agents;
	}
	
	private void aggregateAgents(Context<?> socialContext,
			final MoreRsNetwork<SonomodeAgent, SonomodeEdge> network,
			Collection<SonomodeAgent> agents) {
		// <- LOGGING
		logger.info("Aggregate Agents (base: " + agents.size() + " agents)");
		// LOGGING ->
		
		Map<GimAgentDissimilarityProvider<SonomodeAgent>, Double> map = new HashMap<GimAgentDissimilarityProvider<SonomodeAgent>, Double>();
		map.put(new GimAgentDissimilarityProvider<SonomodeAgent>() {
			@Override
			public double[][] getVertexDissimilarity(
					Map<SonomodeAgent, Integer> agents) {
				return MVertexSimilarityMeasureCalculator
						.getVertexDiceDissimilaritiesR(network.getJungGraph(),
								"all", agents);
			}
		}, ((Double)PmParameterManager.getParameter(GAggregatePa.WEIGHT_FACTOR_NETWORK)).doubleValue());

		map.put(new GGeographyDissimilarityProvider<SonomodeAgent>(GManager
				.getMan().getRootGeography(), ((Double) PmParameterManager
				.getParameter(BasicPa.AGGREGATE_GEO_MAX_DISTANCE))
				.doubleValue()),
			((Double)PmParameterManager.getParameter(GAggregatePa.WEIGHT_FACTOR_GEOGRAPHY)).doubleValue());
		
		map.put(new AgentDissimilarityProvider(), ((Double) PmParameterManager
				.getParameter(GAggregatePa.WEIGHT_FACTOR_AGENT))
				.doubleValue());
		
		new GAggregateAgents<SonomodeAgent, SonomodeEdge>(socialContext,
				network).aggregateAgents(map, agents, 0);
	}
	
	/**
	 * @param agents
	 */
	private void initSocialNetwork(List<SonomodeAgent> agents) {
		Context<SonomodeAgent> socialContext = ModelManager.getMan().getSocialContext();

		// init More
		MManager.init();
		MManager.setSchedule(new MRsSchedule(RunEnvironment.getInstance()
				.getCurrentSchedule()));
		
		MoreRsNetwork<SonomodeAgent, SonomodeEdge> network = networkBuilder.buildNetwork(agents);
		socialContext.addProjection(network);

		// Register network at MoRe
		MNetworkManager.setNetwork(network, ModelManager.SOCNET_NAME);
		
		// build and register blacklist network:
		MoreNetwork<SonomodeAgent, MoreEdge<SonomodeAgent>> blacklistnetwork = new MDirectedNetwork<SonomodeAgent, 
				MoreEdge<SonomodeAgent>>(
				new MDefaultEdgeFactory<SonomodeAgent>(),
				(String) PmParameterManager
						.getParameter(MNetManipulatePa.DYN_BLACKLIST_NAME));
		for(SonomodeAgent agent : agents) {
			blacklistnetwork.addNode(agent);
		}
		MNetworkManager.setNetwork(blacklistnetwork,
				(String) PmParameterManager
						.getParameter(MNetManipulatePa.DYN_BLACKLIST_NAME));
		
		for (SonomodeAgent agent : agents) {
			agent.setMainNetwork(network);
		}
		PmParameterManager.setParameter(MSqlPa.TBLNAME_NETWORK_MEASURES, PmParameterManager.getParameter(SqlPa.TBLNAME_NETWORK_MEASURES));

		// UNDO (but document)
		// init filtered network
		// Context<SonomodeAgent> filteredContext = new
		// DefaultContext<SonomodeAgent>(
		// "FilteredNetwork");
		// socialContext.addSubContext(filteredContext);
		//
		// filteredContext
		// .addProjection((MoreRsNetwork<SonomodeAgent, SonomodeEdge>)
		// MNetworkManager
		// .storeVertexSubnetwork(network,
		// new Predicate<SonomodeAgent>() {
		// @Override
		// public boolean evaluate(SonomodeAgent agent) {
		// return agent.getOpinion() > 0.0;
		// }
		// }, "PositiveOpinion"));
	}
	
	/**
	 * 
	 */
	private void initSocialNetworkAnalysis() {
		logger.debug("Analyse Networks? "
				+ PmParameterManager
						.getParameter(SocNetPa.ANALYSE_NETWORKS_TICK));
		if (((Double) PmParameterManager
				.getParameter(SocNetPa.ANALYSE_NETWORKS_TICK)) != 0.0) {
			SocialNetworkAnalysis socan = new SocialNetworkAnalysis();
			RunEnvironment.getInstance().getCurrentSchedule().schedule(socan);
		}
	}
	
	/**
	 * Called at the end of a simulation.
	 * Calls {@link AbstractSonomodeAgent#reset()}
	 */
	@ScheduledMethod(start = ScheduleParameters.END, priority = ScheduleParameters.LAST_PRIORITY)
	public void reset() {
		//CustomSchedule.inspectSchedule(logger);		
		logger.info("Reset Model");
		
		parameters = null;
		networkBuilder = null;
		
		GManager.reset();
		MManager.reset();
		ModelManager.reset();
		ModelManager.setRunId(ModelManager.getMan().getRunId() + 1);
		PmParameterManager.reset();
		
		try {
			basicInfologger.info("Finished at "
					+ new SimpleDateFormat("HH:mm:ss:SS").format(new Date(System.currentTimeMillis()))
					+ "(taking "
					+ DatatypeFactory.newInstance().newDuration(
							(System.currentTimeMillis() - startTimeMillis)) + ")");
		} catch (DatatypeConfigurationException e) {
			e.printStackTrace();
		}
		AbstractSonomodeAgent.reset();
		DataProvider.disconnect();

		System.gc();
	}
}
