/**
 * 
 */
package de.cesr.sonomode.core;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import repast.simphony.random.RandomHelper;

import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;

import de.cesr.parma.core.PmParameterManager;
import de.cesr.sesamgim.init.agent.GimAgentInitialiseService;
import de.cesr.sonomode.agent.ExtremeSonomodeAgent;
import de.cesr.sonomode.agent.ModerateSonomodeAgent;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.param.RandomPa;
import de.cesr.uranus.core.URandomService;

/**
 * @author Sascha Holzhauer
 *
 */
public class DbAgentInitService implements GimAgentInitialiseService<SonomodeAgent> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(DbAgentInitService.class);
	
	protected int numAgents;
	protected int numExtremists;
	protected int counter = -1;
	
	protected ArrayList<SonomodeAgent> agents;
	
	protected boolean switcher = false;
	protected Uniform rand;
	protected Uniform uniform;
	
	public DbAgentInitService() {
		this.numAgents = ((Integer)PmParameterManager.getParameter(BasicPa.NUM_AGENTS)).intValue();
		this.numExtremists = ((int)(((Double)PmParameterManager.getParameter(BasicPa.PROP_EXTREMISTS)).doubleValue() * numAgents) / 2) * 2;
		this.numExtremists = this.numExtremists < 2 ? 2 : this.numExtremists;
		
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Number of extremists: " + numExtremists + " (BasicPa.PROP_EXTREMISTS: " + 
					((Double)PmParameterManager.getParameter(BasicPa.PROP_EXTREMISTS)).doubleValue() + ")");
		}
		// LOGGING ->

		agents = new ArrayList<SonomodeAgent>(numAgents);
		for (int i = 0; i < numAgents; i++) {
			agents.add(i, null);
		}
		
		this.rand = URandomService.getURandomService().getNewUniformDistribution(
				new MersenneTwister(((Integer)PmParameterManager.getParameter(RandomPa.RANDOM_SEED)).intValue()));
		uniform = (Uniform) RandomHelper
				.getDistribution(ModelManager.RANDOM_OPINION);
		
		for (int i = 0; i < numExtremists; i++) {
			int agent = rand.nextIntFromTo(0, numAgents - 1);
			while (agents.get(agent) != null) {
				agent = rand.nextIntFromTo(0, numAgents - 1);
			}
			if (switcher) {
				agents.add(agent, new ExtremeSonomodeAgent(1, 1));
			} else {
				agents.add(agent, new ExtremeSonomodeAgent(-1, 3));
			}
			switcher = !switcher;
		}
	}

	@Override
	public SonomodeAgent initAgent(int milieu) {
		counter++;
		if (counter > numAgents) {
			throw new IllegalStateException("More than BasicPa.NUM_AGENTS agents were requested!");
		}
		if (agents.get(counter) != null) {
			return agents.get(counter);
		} else {
			return new ModerateSonomodeAgent(uniform.nextDouble(), 2);
		}
	}

}
