/**
 * This file is part of
 * 
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant
 *
 * Copyright (C) 2011 Center for Environmental Systems Research, Kassel, Germany
 * 
 * SesamGIM - Geographical Initialisation for Milieu Agents is free software:
 * You can redistribute it and/or modify it under the terms of the GNU General 
 * Public License as published by the Free Software Foundation, either 
 * version 3 of the License, or (at your option) any later version.
 *  
 * SoNOMoDe - Social Networks in Opinion Dynamics Models of Deffuant is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Sascha Holzhauer
 * Center for Environmental Systems Research, Kassel
 */
package de.cesr.sonomode.core;

import java.text.DecimalFormat;
import java.text.NumberFormat;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.data.db.outputter.DefaultDBOutputter;
import repast.simphony.freezedry.datasource.DBConnectionInfo;
import de.cesr.more.util.MoreRunIdProvider;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.parma.definition.PmFrameworkPa;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.param.SqlPa;

/**
 * @author holzhauer
 *
 */
public class ModelManager implements MoreRunIdProvider {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(ModelManager.class);

	public static final String RANDOM_OPINION = "Random opinion";
	
	public static final String RANDOM_NOISE = "Random noise";
	
	public static final String RANDOM_NOISE_AMOUNT = "Random noise amount";
	
	
	public static final String SOCNET_NAME = "SocialNetwork";
	
	public static final String SOC_CONTEXT_NAME = "SocialContext";
	
	public static ModelManager man;
	
	/**
	 * {@link NumberFormat} to format integer numbers
	 */
	protected NumberFormat		integerFormat;

	/**
	 * {@link NumberFormat} to format floating point numbers
	 */
	protected NumberFormat		floatPointFormat;
	
	private static int runId = Integer.MIN_VALUE;
	
	Context<SonomodeAgent> socialContext;
	
	public ModelManager() {
		this.integerFormat = new DecimalFormat("0000");
		this.floatPointFormat = new DecimalFormat("0.0000");		
	}
	
	
	public Context<SonomodeAgent> getSocialContext() {
		return this.socialContext;
	}
	
	public void setSocialContext(Context<SonomodeAgent> socialContext) {
		this.socialContext = socialContext;
	}
	
	public static ModelManager getMan() {
		if (man == null) {
			man = new ModelManager();
		}
		return man;
	}

	@Override
	public int getRunId() {
		DBConnectionInfo dbConInfo = new DBConnectionInfo(
				(String)PmParameterManager.getParameter(PmFrameworkPa.USER),
				(String)PmParameterManager.getParameter(PmFrameworkPa.PASSWORD),
				"jdbc:mysql://" + (String)PmParameterManager.getParameter(PmFrameworkPa.LOCATION) + ":3306/" + (String)PmParameterManager.getParameter(PmFrameworkPa.DBNAME),
				"");
		int tmpRunID = DefaultDBOutputter.getCurrentMaxRunId(((String)PmParameterManager.getParameter(SqlPa.TBLNAME_RUN_INFO)), dbConInfo);
		if (tmpRunID > runId) {
			runId = tmpRunID;
		}
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("DefaultDBOutputter: " + tmpRunID);
			logger.debug("Get runId: " + runId);
		}
		// LOGGING ->

		return runId;
	}
	
	public static void setRunId(int runId) {
		ModelManager.runId = runId;
	}
	
	/**
	 * @see de.cesr.lara.components.model.LaraModel#getFloatPointFormat()
	 */
	public NumberFormat getFloatPointFormat() {
		return this.floatPointFormat;
	}

	/**
	 * @see de.cesr.lara.components.model.LaraModel#getIntegerFormat()
	 */
	public NumberFormat getIntegerFormat() {
		return this.integerFormat;
	}
	
	public static void reset() {
		man = null;
	}
}
