/**
 * 
 */
package de.cesr.sonomode.function;

import org.apache.log4j.Logger;

/**
 * @author Sascha Holzhauer
 *
 */
public class Ra2002KernelFunction implements KernelFunction {

	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(Ra2002KernelFunction.class);

	private Ra2002KernelFunction() {};
	
	static KernelFunction function = null;
	
	static public KernelFunction getInstance() {
		if (function == null) {
			function = new Ra2002KernelFunction();
		}
		return function;
	}
	
	/**
	 * @see de.cesr.sonomode.function.KernelFunction#getValue(double,
	 *      double, double, double)
	 */
	@Override
	public double getValue(double opinionOwn, double opinionOther,
			double uncertaintyOwn, double uncertaintyOther) {
		double result;
		double v = Math.min(opinionOwn + uncertaintyOwn, opinionOther
				+ uncertaintyOther)
				- Math.max(opinionOwn - uncertaintyOwn, opinionOther
						- uncertaintyOther);

		if (v > 0.0) {
			result = v / uncertaintyOther - 1;
		} else {
			result = 0.0;
		}

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("KernelFunction: " + result + " | v: " + v
					+ "( non-overlapping part: " + (2.0 * uncertaintyOther - v)
					+ ")");
		}
		// LOGGING ->

		return result;
	}
}
