/**
 * 
 */
package de.cesr.sonomode.function;

import org.apache.log4j.Logger;

/**
 * 
 * "[...] individuals take into account the uncertainty of their interlocutor, and interlocutors with
 * a low uncertainty (high confidence) tend to be more influential than those with a high uncertainty."
 * 
 * "The value of this function is 1 when the segment [x'-u',x'+u'] is totally included in the segment [x-u, x+u].
 * Otherwise, it is lower than 1."
 * 
 * @author holzhauer
 *
 */
public class RaKernelFunction implements KernelFunction {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(RaKernelFunction.class);

	private RaKernelFunction() {};
	
	static KernelFunction function = null;
	
	static public KernelFunction getInstance() {
		if (function == null) {
			function = new RaKernelFunction();
		}
		return function;
	}
	
	/**
	 * @see de.cesr.sonomode.function.KernelFunction#getValue(double, double, double, double)
	 */
	@Override
	public double getValue(double opinionOwn, double opinionOther,
			double uncertaintyOwn, double uncertaintyOther) {
		double result;
		double v = Math.min(opinionOwn + uncertaintyOwn, opinionOther + uncertaintyOther) -
					Math.max(opinionOwn - uncertaintyOwn, opinionOther - uncertaintyOther);
		if (v > 0.0) {
			 result = v / (2.0 * uncertaintyOther);
		} else {
			 result = 0.0;
		}
		
		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("KernelFunction: " + result + " | v: " + v);
		}
		// LOGGING ->

		return result;
	}
}
