/**
 * 
 */
package de.cesr.sonomode.param;

import repast.simphony.engine.schedule.ScheduleParameters;
import de.cesr.parma.core.PmParameterDefinition;
import de.cesr.sonomode.function.BcKernelFunction;

/**
 * @author holzhauer
 *
 */
public enum BasicPa implements PmParameterDefinition {
	
	PARAM_SET_ID(Integer.class, 123005),
		
	NUM_AGENTS(Integer.class, 100),
	
	END_TICK(Integer.class, 300),
	
	PROP_EXTREMISTS(Double.class, 0.01),
	
	KERNEL_FUNCTION(Class.class, BcKernelFunction.class),
	
	EPSILON(Double.class, 0.01),
	
	AGENT_CONVERGENCE_THRESHOLD(Double.class, 0.01),
	
	AGGREGATE_AGENT_AMOUNT(Integer.class, 0),

	AGGREGATE_GEO_MAX_DISTANCE(Double.class, 3333.33),
	
	USE_DB_AGENT_INITIALISER(Boolean.class, Boolean.TRUE),

	/**
	 * Use spatially differentiated initial opinion with given i. 0 to omit.
	 */
	SPATIAL_INITIAL_OPINION_ID(Integer.class, 0),

	/**
	 * Location of XML file that specifies parameters
	 */
	XML_PARAMETER_FILE(String.class, "./config/param/Parameters.xml"),
	
	/**
	 * Updating of opinion and uncertainty of the whole population.
	 */
	ASSYNCHRON_POP_UPDATE(Boolean.class, Boolean.TRUE),
	
	/**
	 * Updating of opinion and uncertainty of partners
	 * during an interaction. Setting to TRUE seems implausible.
	 * Applies only when ASSYNCHRON_POP_UPDATE is true.
	 */
	ASSYNCHRON_UPDATE(Boolean.class, Boolean.FALSE),
	
	INIT_ALLOW_REDRAWS(Boolean.class, Boolean.TRUE),
	
	AREAS_SHAPEFILE(String.class, "config/shapes/area/areaShapes04.shp"),

	/**
	 * Time step for output of shape files (0 to omit; negative values are
	 * interpreted as interval starting at 0). ScheduleParameters.END = Infinity is possible
	 */
	WRITE_SHAPEFILES(Double.class, ScheduleParameters.END),
	
	SHAPEFILE_OUTPUT_FOLDER(String.class, "output/shapes");

	private Class < ? >	type;
	private Object		defaultValue;

	BasicPa(Class<?> type) {
		this(type, null);
	}

	BasicPa(Class<?> type, Object defaultValue) {
		this.type = type;
		this.defaultValue = defaultValue;
	}

	@Override
	public Class < ? > getType() {
		return type;
	}

	@Override
	public Object getDefaultValue() {
		return defaultValue;
	}
}
