/**
 * 
 */
package de.cesr.sonomode.param;

import de.cesr.parma.core.PmParameterDefinition;
import de.cesr.parma.core.PmParameterManager;

/**
 * @author holzhauer
 *
 */
public enum RandomPa implements PmParameterDefinition {

	/**
	 * Applications:
	 * - Agent shuffling
	 * - RS parameter seed
	 */
	RANDOM_SEED(Integer.class, 0),
	
	RANDOM_SEED_OPINION(Integer.class, 0),
	
	RANDOM_SEED_NOISE(Integer.class, 0),

	/**
	 * Random seed used for random stream used for initialising networks (for
	 * instance, rewiring)
	 */
	RANDOM_SEED_NETWORKINIT(Integer.class, 0),

	/**
	 * In the edge manager, new links a created according to probabilistic
	 * choice with respect to opinion difference.
	 */
	RANDNOM_SEED_DYN_EDGE_CREATING(Integer.class, RANDOM_SEED_NETWORKINIT),

	NOISE_TRESHOLD(Double.class, 0.1),
	
	NOISE_AMOUNT(Double.class, 0.3),
	
	RANDOM_SEED_NOISE_AMOUNT(Integer.class, 0);
	

	private Class < ? >	type;
	private Object		defaultValue;

	RandomPa(Class<?> type) {
		this(type, null);
	}

	RandomPa(Class<?> type, Object defaultValue) {
		this.type = type;
		this.defaultValue = defaultValue;
	}

	RandomPa(Class<?> type, PmParameterDefinition defaultDefinition) {
		this.type = type;
		if (defaultDefinition != null) {
			this.defaultValue = defaultDefinition.getDefaultValue();
			PmParameterManager.setDefaultParameterDef(this, defaultDefinition);
		} else {
			this.defaultValue = null;
		}
	}

	@Override
	public Class < ? > getType() {
		return type;
	}

	@Override
	public Object getDefaultValue() {
		return defaultValue;
	}

}
