/**
 * 
 */
package de.cesr.sonomode.param;

import de.cesr.more.rs.building.MGeoRsLattice2DNetworkBuilder;
import de.cesr.parma.core.PmParameterDefinition;
import de.cesr.sonomode.socnet.builder.RandomNetworkBuilder;
import de.cesr.sonomode.socnet.manipulate.SonomodeEdgeManager;
import de.cesr.sonomode.socnet.manipulate.SonomodeEdgeWeightUpdater;

/**
 * @author holzhauer
 *
 */
public enum SocNetPa implements PmParameterDefinition {
	
	SOCNET_NAME(String.class, "SocialNetwork"),
	
	SOCNET_PARAM_ID(Integer.class, 1),

	/**
	 * Radius within to search for potential partners in m (!)
	 */
	SEARCH_RADIUS(Double.class, 200.0),

	/**
	 * Distance to expand the radius in m (!)
	 */
	X_SEARCH_RADIUS(Double.class, 100.0),

	/**
	 * Max radius within to search for potential partners in m (!)
	 */
	MAX_SEARCH_RADIUS(Double.class, 500.0),
	
	/**
	 * Geographical distance dimension weight regarding social distance attachment
	 * network builder 
	 */
	DIM_WEIGHTS_GEO(Double.class, 0.5),
	
	/**
	 * Milieu preference dimension weight regarding social distance attachment
	 * network builder 
	 */
	DIM_WEIGHTS_MILIEU(Double.class, 0.5),
	
	
	/**
	 * Intensity with which to shuffle agents in ContextCreator before the network
	 * is built (0 > no shuffle; 10 > complete shuffle)
	 */
	SHUFFLE_AGENTS_BEFORE_NET_BUILT(Integer.class, 10),

	/**
	 * File to read agent network from
	 */
	SOURCE_FILE(String.class, "data/network/smallworld.graphml"),

	/**
	 * Time step for output of network to NETWORK_OUTPUT_FOLDER
	 * (0 to omit; negative values are interpreted as interval starting at 0).
	 */
	OUTPUT_NETWORK(Integer.class, 0),
	
	/**
	 * File to store a newly initiated social network to
	 */
	NETWORK_OUTPUT_FOLDER(String.class, "output/networks/"),
	
	/**
	 * Used by {@link RandomNetworkBuilder}.
	 */
	RANDOM_DENSITY(Double.class, 0.1),

	/**
	 * Time step for output of shape files (0.0 to omit; negative values are
	 * interpreted as interval starting at 2.0). ScheduleParameters.END =
	 * Infinity is possible
	 */
	ANALYSE_NETWORKS_TICK(Double.class, 0.0),
	
	NETWORK_BUILDER(Class.class, MGeoRsLattice2DNetworkBuilder.class),

	/**
	 * Initialisation of edge weights.
	 */
	DISTRIBUTE_EDGE_WEIGHTS_UNIFORMLY(Boolean.class, Boolean.TRUE),

	/**
	 * Needs to implement MoreEgoNetworkProcessor.
	 */
	DYN_EDGE_WEIGHT_UPDATER(Class.class, SonomodeEdgeWeightUpdater.class),
	
	/**
	 * Needs to implement MoreEgoNetworkProcessor.
	 */
	DYN_EDGE_MANAGER(Class.class, SonomodeEdgeManager.class);
	
	private Class < ? >	type;
	private Object		defaultValue;

	SocNetPa(Class<?> type) {
		this(type, null);
	}

	SocNetPa(Class<?> type, Object defaultValue) {
		this.type = type;
		this.defaultValue = defaultValue;
	}

	@Override
	public Class < ? > getType() {
		return type;
	}

	@Override
	public Object getDefaultValue() {
		return defaultValue;
	}

}
