/**
 *
 * Center for Environmental Systems Research, Kassel
 * Created by Sascha Holzhauer on 30.07.2010
 */
package de.cesr.sonomode.param.reader;



import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;

import de.cesr.parma.core.PmAbstractParameterReader;
import de.cesr.parma.core.PmParameterDefinition;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.param.MilieuPa;
import de.cesr.sonomode.param.SqlPa;
import de.cesr.sonomode.util.DataProvider;
import de.cesr.sonomode.util.MilieuPrefParameterMap;




/**
 * 
 * @author Sascha Holzhauer
 * @date 30.07.2010
 * 
 */
public class MilieuPrefDataReader extends PmAbstractParameterReader {

	/**
	 * Logger
	 */
	static private Logger	logger	= Logger.getLogger(MilieuPrefDataReader.class);

	/**
	 * @see param.framework.ParameterReader#initParameters()
	 */
	@Override
	public void initParameters() {
		MilieuPrefParameterMap map = new MilieuPrefParameterMap();
		String t1 = (String) PmParameterManager.getParameter(SqlPa.TBLNAME_MILIEU_PREFERENCES);

		// fetch number of households along milieu groups (not milieus!):
		String sql = "SELECT * FROM " + t1 + " AS t1 " + "WHERE id ="
				+ PmParameterManager.getParameter(MilieuPa.MILIEU_PREF_PARAMID) + ";";

		if (logger.isDebugEnabled()) {
			logger.debug("MySQL-Satement in init(): " + sql);
		}

		ResultSet result = DataProvider.getInstance().connect(sql);
		try {
			boolean hasNext = result.next();
			if (!hasNext) {
				logger.error("No milieu preference parameter set in table for paramID "
						+ PmParameterManager.getParameter(MilieuPa.MILIEU_PREF_PARAMID));
				throw new IllegalStateException("No milieu preference parameter set in table for paramID "
						+ PmParameterManager.getParameter(MilieuPa.MILIEU_PREF_PARAMID));
			}
			
			String param_class = null, param_name = null;

			while (hasNext) {
				int milieu = result.getInt("milieu_id");
				for (int i = 1; i <= result.getMetaData().getColumnCount(); i++) {
					if (result.getMetaData().getColumnName(i).contains(":")) {					
						try {
							param_class = result.getMetaData().getColumnName(i).split(":")[0];
							param_name = result.getMetaData().getColumnName(i).split(":")[1];
							map.setPref(milieu, (PmParameterDefinition) Enum.valueOf((Class<Enum>) Class
									.forName(param_class), param_name), result.getObject(i));
							
							// <- LOGGING
							if (logger.isDebugEnabled()) {
								logger.debug("Read milieu preference " + param_class + ":" + param_name + ": " + result.getObject(i));
							}
							// LOGGING ->

						} catch (ClassNotFoundException e) {
							logger.error("Class for param: " + param_class + "." + param_name + " not found!");
							e.printStackTrace();
						}
					}
					
				}
				hasNext = result.next();
			}
			result.close();
			PmParameterManager.setParameter(MilieuPa.MILIEU_PREF_PARAMS, map);
		}
		catch (SQLException e) {
			logger.error("Error in fetching milieu net data");
			e.printStackTrace();
		}
	}
}
