/**
 * 
 */
package de.cesr.sonomode.socnet.builder;

import java.util.Collection;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.space.gis.Geography;
import repast.simphony.space.graph.UndirectedJungNetwork;
import de.cesr.more.building.edge.MoreEdgeFactory;
import de.cesr.more.rs.building.MoreRsNetworkBuilder;
import de.cesr.more.rs.building.edge.MRsEdgeFactory;
import de.cesr.more.rs.edge.MRepastEdge;
import de.cesr.more.rs.network.MRsContextJungNetwork;
import de.cesr.more.rs.network.MoreRsNetwork;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.param.SocNetPa;

/**
 * @author holzhauer
 *
 */
public class CompleteNetworkBuilder<AgentType, EdgeType extends MRepastEdge<AgentType>> 
	implements MoreRsNetworkBuilder<AgentType, EdgeType> {
	
	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(CompleteNetworkBuilder.class);

	Context<AgentType> context;
	MoreEdgeFactory<AgentType, EdgeType> eFac;
	
	/**
	 * @param geography
	 * @deprecated
	 * 
	 */
	public CompleteNetworkBuilder(Geography<Object> geography) {
	}
	
	/**
	 * @param eFac
	 */
	public CompleteNetworkBuilder(MRsEdgeFactory<AgentType, EdgeType> eFac) {
		this.eFac = eFac;
	}
	
	@Override
	public MoreRsNetwork<AgentType, EdgeType> buildNetwork(
			Collection<AgentType> arg0) {
		if (context == null) {
			// <- LOGGING
			logger.error("The context has not been set!");
			// LOGGING ->
			throw new IllegalStateException("The context has not bee set!");
		}
		
		MRsContextJungNetwork<AgentType, EdgeType> network = new MRsContextJungNetwork<AgentType, EdgeType >(
				new UndirectedJungNetwork<AgentType>(ModelManager.SOCNET_NAME), context);
		for (AgentType agent : context) {
			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("Add agent " + agent + " to network.");
			}
			// LOGGING ->

			network.addNode(agent);
			
			// connect this agent with every already added other (undirected):
			for (AgentType other : network.getNodes()) {
				if (other != agent) {
					network.connect(eFac.createEdge(agent, other, false));
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						logger.debug(agent + "> connect to " + other);
					}
					// LOGGING ->
				}
			}
		}
		
		return (MoreRsNetwork<AgentType, EdgeType>) network;
	}


	public void setContext(Context<AgentType> context) {
		this.context = context;	
	}
}
