/**
 * 
 */
package de.cesr.sonomode.socnet.builder;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.collections15.Transformer;

import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.rs.edge.MRepastEdge;
import de.cesr.more.util.io.MoreIoUtilities;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.socnet.SonomodeEdge;
import edu.uci.ics.jung.io.GraphMLMetadata;

/**
 * @author holzhauer
 *
 */
public class NetworkUtils  {

	/**
	 * @param network
	 * @param file
	 */
	static public <AgentType extends SonomodeAgent, EdgeType extends SonomodeEdge> void outputNetwork(
			MoreNetwork<AgentType, EdgeType> network, File file) {
		
		Map<String, GraphMLMetadata<AgentType>> metaData = new HashMap<String, GraphMLMetadata<AgentType>>();
		metaData.put("Opinion", new GraphMLMetadata<AgentType>("Opinion[float]", "0.0", new Transformer<AgentType, String>() {
			@Override
			public String transform(SonomodeAgent agent) {
				return new Double(agent.getOpinion()).toString();
			}
		}));
		metaData.put("Uncertainty", new GraphMLMetadata<AgentType>("Uncertainty[float]", "0.0", new Transformer<AgentType, String>() {
			@Override
			public String transform(SonomodeAgent agent) {
				return new Double(agent.getUncertaint()).toString();
			}
		}));
		metaData.put("NumNewLinks", new GraphMLMetadata<AgentType>("NumNewLinks[int]", "0.0", new Transformer<AgentType, String>() {
			@Override
			public String transform(SonomodeAgent agent) {
				return new Double(agent.getNumNewLinks()).toString();
			}
		}));
		
		Map<String, GraphMLMetadata<EdgeType>> edgeMetaData = new HashMap<String, GraphMLMetadata<EdgeType>>();
		edgeMetaData.put("Activity", new GraphMLMetadata<EdgeType>("Activity[boolean]", "0", new Transformer<EdgeType, String>() {
			@Override
			public String transform(EdgeType edge) {
				return (edge.isActive() ? "1" : "0"); 
			}
		}));
		edgeMetaData.put("Weight", new GraphMLMetadata<EdgeType>("Weight[double]", "0.0", new Transformer<EdgeType, String>() {
			@Override
			public String transform(EdgeType edge) {
				return new Double(edge.getWeight()).toString(); 
			}
		}));
		edgeMetaData.put("Change", new GraphMLMetadata<EdgeType>("Change[double]", "0.0", new Transformer<EdgeType, String>() {
			@Override
			public String transform(EdgeType edge) {
				return new Double(edge.getChange()).toString(); 
			}
		}));
		MoreIoUtilities.<AgentType, EdgeType>outputGraph(network, file, metaData, edgeMetaData);
	}
}
