/**
 * 
 */
package de.cesr.sonomode.socnet.builder;

import java.util.Collection;

import org.apache.log4j.Logger;

import repast.simphony.context.Context;
import repast.simphony.context.space.graph.RandomDensityGenerator;
import repast.simphony.space.gis.Geography;
import repast.simphony.space.graph.UndirectedJungNetwork;
import de.cesr.more.building.edge.MoreEdgeFactory;
import de.cesr.more.rs.building.MoreRsNetworkBuilder;
import de.cesr.more.rs.building.edge.MRsEdgeFactory;
import de.cesr.more.rs.edge.MRepastEdge;
import de.cesr.more.rs.network.MRsContextJungNetwork;
import de.cesr.more.rs.network.MoreRsNetwork;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.param.SocNetPa;

/**
 * @author holzhauer
 * TODO incorporate edgeFactory!
 */
public class RandomNetworkBuilder <AgentType, EdgeType extends MRepastEdge<AgentType>> extends RandomDensityGenerator<AgentType> implements
MoreRsNetworkBuilder<AgentType, EdgeType> {

	/**
	 * Logger
	 */
	static private Logger logger = Logger.getLogger(RandomNetworkBuilder.class);

	Context<AgentType> context;
	MoreEdgeFactory<AgentType, EdgeType> eFac;
	
	/**
	 * @param geography
	 * @deprecated
	 * 
	 */
	public RandomNetworkBuilder(Geography<Object> geography) {
		super(((Double)PmParameterManager.getParameter(SocNetPa.RANDOM_DENSITY)).doubleValue(), false, true);
	}
	
	/**
	 * @param eFac
	 */
	public RandomNetworkBuilder(MRsEdgeFactory<AgentType, EdgeType> eFac) {
		super(((Double)PmParameterManager.getParameter(SocNetPa.RANDOM_DENSITY)).doubleValue(), false, true);
		this.eFac = eFac;
	}

	
	public RandomNetworkBuilder(MRsEdgeFactory<AgentType, EdgeType> eFac, boolean allowSelfLoops, boolean symmetric) {
		super(((Double)PmParameterManager.getParameter(SocNetPa.RANDOM_DENSITY)).doubleValue(), allowSelfLoops, symmetric);
	}

	@SuppressWarnings("unchecked")
	@Override
	public MoreRsNetwork<AgentType, EdgeType> buildNetwork(Collection<AgentType> agents) {
		
		if (context == null) {
			// <- LOGGING
			logger.error("The context has not bee set!");
			// LOGGING ->
			throw new IllegalStateException("The context has not bee set!");
		}
		
		MRsContextJungNetwork<AgentType, EdgeType> network = new MRsContextJungNetwork<AgentType, EdgeType >(
				new UndirectedJungNetwork<AgentType>(ModelManager.SOCNET_NAME), context);
		for (AgentType agent : context) {
			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("Add agent " + agent + " to network.");
			}
			// LOGGING ->

			network.addNode(agent);
		}
		
		return (MoreRsNetwork<AgentType, EdgeType>) super.createNetwork(network);
	}

	/**
	 * @see de.cesr.more.rs.building.MoreRsNetworkBuilder#setContext(repast.simphony.context.Context)
	 */
	@Override
	public void setContext(Context<AgentType> context) {
		this.context = context;
	}
}
