/**
 * 
 */
package de.cesr.sonomode.socnet.manipulate;

import java.lang.reflect.InvocationTargetException;

import org.apache.log4j.Logger;

import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.manipulate.agent.MoreEgoNetworkProcessor;
import de.cesr.more.param.MMilieuNetworkParameterMap;
import de.cesr.more.param.MNetworkBuildingPa;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.core.ModelManager;
import de.cesr.sonomode.function.KernelFunction;
import de.cesr.sonomode.param.BasicPa;
import de.cesr.sonomode.socnet.SonomodeEdge;

/**
 * @author Sascha Holzhauer
 *
 */
public class SonomodeEdgeWeightUpdater implements
		MoreEgoNetworkProcessor<SonomodeAgent, SonomodeEdge> {

	protected static final double INCREASE_FACTOR = 0.1;
	protected static final double WEIGHT_MAX = 2.0;
	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(SonomodeEdgeWeightUpdater.class);

	protected KernelFunction function = null;

	protected KernelFunction getKernelFunction() {

		if (function == null) {
			try {
				try {
					function = (KernelFunction) ((Class<?>) PmParameterManager.getParameter(BasicPa.KERNEL_FUNCTION)).getMethod("getInstance").invoke(null);
				} catch (NoSuchMethodException e) {
		
					function = (KernelFunction) ((Class<?>) PmParameterManager.getParameter(BasicPa.KERNEL_FUNCTION)).
							getConstructor().newInstance();
				}
				
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InstantiationException e1) {
				e1.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			}
		}
		return function;
	}

	@Override
	public void process(SonomodeAgent agent,
			MoreNetwork<SonomodeAgent, SonomodeEdge> network) {

		double decreaseValue = ((MMilieuNetworkParameterMap) PmParameterManager
				.getParameter(MNetworkBuildingPa.MILIEU_NETWORK_PARAMS))
				.getDynDecreaseAmount(agent.getMilieuGroup());

		double opinionAgent = agent.getOpinion();
		double uncertaintAgent = agent.getUncertaint();
		for (SonomodeAgent neighbour : network.getPredecessors(agent)) {
			SonomodeEdge edge = network.getEdge(neighbour, agent);

			double opinionNeighbour = neighbour.getOpinion();

			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug(agent + "> Ego: " + agent.getOpinion() + " ("
						+ agent.getUncertaint() + ") - Alter: "
						+ opinionNeighbour + " (" + neighbour.getUncertaint()
						+ ")");
			}
			// LOGGING ->

			if (Math.abs(opinionAgent - opinionNeighbour) > uncertaintAgent
					&& Math.abs(opinionAgent - opinionNeighbour) > neighbour
							.getUncertaint()) {
				// partner's opinion is outside ego's uncertainty range and
				// ego's opinion is outside other's uncertainty range
				edge.setWeight(edge.getWeight() - decreaseValue);

				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Edge ("
							+ edge
							+ ") weight decreased by "
							+ ModelManager.getMan().getFloatPointFormat()
									.format(decreaseValue) + " ( new value: "
							+ edge.getWeight() + ")");
				}
				// LOGGING ->

			} else {
				// otherwise increase weight by
				// <kernel-function>(ego,other)+<kernel-function>(other,ego)
				if (edge.getWeight() < WEIGHT_MAX) {
					function = getKernelFunction();
	
					double weightChange = (function.getValue(opinionAgent, opinionNeighbour,
							uncertaintAgent, neighbour.getUncertaint())
					+ function.getValue(opinionNeighbour, opinionAgent,
									neighbour.getUncertaint(),
									uncertaintAgent)) * INCREASE_FACTOR;
					edge.setWeight(edge.getWeight()
							+ weightChange);
	
					// <- LOGGING
					if (logger.isDebugEnabled()) {
						if (weightChange > 0.0) {
							logger.debug("Edge ("
									+ edge
									+ ") weight increased by "
									+ ModelManager
											.getMan()
											.getFloatPointFormat()
											.format(weightChange)
									+ " ( new value: " + edge.getWeight() + ")");
						}
					}
					// LOGGING ->
				}
			}
		}
	}
}
