/**
 * 
 */
package de.cesr.sonomode.socnet.manipulate;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.apache.log4j.Logger;

import repast.simphony.space.gis.Geography;
import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;
import de.cesr.more.basic.MManager;
import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.manipulate.edge.MoreNetworkEdgeModifier;
import de.cesr.more.param.MRandomPa;
import de.cesr.more.rs.geo.util.MGeographyWrapper;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.socnet.SonomodeEdge;

/**
 * @author Sascha Holzhauer
 */
public class SonomodeProbEdgeManager extends SonomodeEdgeManager {

	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(SonomodeProbEdgeManager.class);

	protected static final double MAX_ROULETTE_PART_VALUE = 100.0;
	
	protected Uniform randProbabilisticEdgeCreation;

	public SonomodeProbEdgeManager(
			MoreNetworkEdgeModifier<SonomodeAgent, SonomodeEdge> edgeMan,
			Geography<Object> geography) {
		this(edgeMan);
		this.geoWrapper = new MGeographyWrapper<Object>(geography);
	}

	public SonomodeProbEdgeManager(
			MoreNetworkEdgeModifier<SonomodeAgent, SonomodeEdge> edgeMan) {
		super(edgeMan);

		// register separate random number generator:
		MManager.getURandomService().registerGenerator(
				RANDOM_GENERATOR_PROBABILISTIC_EDGE_CREATION,
				new MersenneTwister(((Integer) PmParameterManager
						.getParameter(MRandomPa.RANDOM_SEED_NETWORK_BUILDING))
						.intValue()));
		MManager.getURandomService().registerDistribution(
				new Uniform(MManager.getURandomService().getGenerator(
						RANDOM_GENERATOR_PROBABILISTIC_EDGE_CREATION)),
				RANDOM_DIST_PROBABILISTIC_EDGE_CREATION);

		randProbabilisticEdgeCreation = (Uniform) MManager.getURandomService()
				.getDistribution(RANDOM_DIST_PROBABILISTIC_EDGE_CREATION);
	}

	

	/**
	 * @see de.cesr.more.manipulate.agent.MBlacklistThresholdLinkProcessor#createNewLinks(int,
	 *      A, de.cesr.more.basic.network.MoreNetwork, java.util.Map,
	 *      de.cesr.more.manipulate.agent.MBlacklistThresholdLinkProcessor.TieCounter,
	 *      java.util.ArrayList, java.util.ArrayList, java.util.ArrayList)
	 */
	@Override
	protected int createNewLinks(int numNewConnections, SonomodeAgent agent,
			MoreNetwork<SonomodeAgent, SonomodeEdge> net,
			Map<Double, Set<SonomodeAgent>> potPartners,
			TieCounter tieCounter, ArrayList<SonomodeAgent> transitiveAgents,
			ArrayList<SonomodeAgent> reciprocalAgents,
			ArrayList<SonomodeAgent> localAgents) {

		// create roulette wheel:
		Map<SonomodeAgent, BigDecimal> roulette_wheel = new LinkedHashMap<SonomodeAgent, BigDecimal>();
		BigDecimal sum = new BigDecimal(0.0);
		double part;
		for (Entry<Double, Set<SonomodeAgent>> entry : potPartners.entrySet()) {
			for (SonomodeAgent potPartner : entry.getValue()) {
				// calculate reciprocal value of difference in opinion ((shift to left by 1.0):
				part = (entry.getKey() + 1.0) * MAX_ROULETTE_PART_VALUE > 1.0 ?  1.0 / (entry.getKey() + 1.0) : MAX_ROULETTE_PART_VALUE;
				
				// consider certain level of perfectionism:
				BigDecimal partBd = new BigDecimal(Math.pow(part, netParams.getDynEdgeManageOptimum(agent.getMilieuGroup())));
				
				if (part >= Double.MAX_VALUE ) {
					logger.error("Roulette wheel part reached Double.MAX_VALUE!!");
				}
				
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Roulette part for " + potPartner + ": " + partBd.doubleValue());
				}
				// LOGGING ->

				roulette_wheel.put(potPartner, partBd);
				sum = sum.add(partBd);
				
				if (sum.doubleValue() == Double.POSITIVE_INFINITY) {
					logger.warn("Sum reached Double.POSITIVE_INFINITY!");
				}
			}
		}

		int counter = 0;
		double randFloat = randProbabilisticEdgeCreation.nextDoubleFromTo(0.0,
				sum.doubleValue());

		// play
		while (roulette_wheel.size() > 0 && counter < numNewConnections) {
			BigDecimal removedSum = null;
			Set<SonomodeAgent> toRemove = new HashSet<SonomodeAgent>();
			BigDecimal pointer = new BigDecimal(0.0);
			for (Entry<SonomodeAgent, BigDecimal> entry : roulette_wheel.entrySet()) {
				pointer = pointer.add(entry.getValue());
				if (pointer.doubleValue() >= randFloat) {
					if (!net.isSuccessor(agent, entry.getKey())
							&& entry.getKey() != agent
							&& !getBlacklistNetwork().isSuccessor(agent,
									entry.getKey())) {
						edgeMan.createEdge(net, entry.getKey(), agent);

						// <- LOGGING
						if (logger.isDebugEnabled()) {
							logger.debug("Edge created: "
									+ net.getEdge(entry.getKey(), agent));
						}
			
						if (Logger
								.getLogger(
										"de.cesr.more.manipulate.agent.MBlacklistThresholdLinkProcessor.links")
								.isInfoEnabled()) {
							if (transitiveAgents.contains(entry.getKey())) {
								tieCounter.transitiveTies++;
							}
							if (reciprocalAgents.contains(entry.getKey())) {
								tieCounter.reciprocalTies++;
							}
							if (localAgents.contains(entry.getKey())) {
								tieCounter.localTies++;
							}
						}
						counter++;
						// LOGGING ->
					}
					toRemove.add(entry.getKey());
					removedSum = entry.getValue();
					break;
				}
			}
			
			if (toRemove.size() == 0) {
				logger.error("No partner identified from roulette wheel!");
				throw new IllegalStateException("No partner identified from roulette wheel!");
			}
			for (SonomodeAgent remove : toRemove) {
				roulette_wheel.remove(remove);
			}
			sum = sum.subtract(removedSum);
			randFloat = randProbabilisticEdgeCreation
					.nextDoubleFromTo(0.0, sum.doubleValue());
		}
		return counter;
	}
}
