/**
 * 
 */
package de.cesr.sonomode.socnet.manipulate;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import repast.simphony.space.gis.Geography;
import cern.jet.random.AbstractDistribution;
import cern.jet.random.Uniform;
import de.cesr.more.basic.MManager;
import de.cesr.more.basic.network.MoreNetwork;
import de.cesr.more.manipulate.agent.MThresholdLinkProcessor;
import de.cesr.more.manipulate.edge.MoreNetworkEdgeModifier;
import de.cesr.more.param.MRandomPa;
import de.cesr.parma.core.PmParameterManager;
import de.cesr.sonomode.agent.SonomodeAgent;
import de.cesr.sonomode.socnet.SonomodeEdge;

/**
 * @author Sascha Holzhauer
 *
 */
public class SonomodeRandomEdgeManager extends
		MThresholdLinkProcessor<SonomodeAgent, SonomodeEdge> {

	/**
	 * Logger
	 */
	static private Logger logger = Logger
			.getLogger(SonomodeRandomEdgeManager.class);

	protected Uniform rand;

	/**
	 * Dummy constructor...
	 * 
	 * @param edgeMan
	 * @param geography
	 */
	public SonomodeRandomEdgeManager(
			MoreNetworkEdgeModifier<SonomodeAgent, SonomodeEdge> edgeMan,
			Geography<Object> geography) {
		this(edgeMan);
	}

	public SonomodeRandomEdgeManager(
			MoreNetworkEdgeModifier<SonomodeAgent, SonomodeEdge> edgeMan) {
		super(edgeMan);
		this.edgeMan = edgeMan;
		AbstractDistribution abstractDis = MManager
				.getURandomService()
				.getDistribution(
						(String) PmParameterManager
								.getParameter(MRandomPa.RND_UNIFORM_DIST_NETWORK_BUILDING));

		if (abstractDis instanceof Uniform) {
			this.rand = (Uniform) abstractDis;
		} else {
			this.rand = MManager.getURandomService().getUniform();
			logger.warn("Use default uniform distribution");
		}

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("...SonomodeRandomEdgeManager initialised.");
		}
		// LOGGING ->
	}

	/**
	 * @param numNewConnections
	 */
	@Override
	public void makeNewConnections(int numNewConnections, SonomodeAgent agent,
			MoreNetwork<SonomodeAgent, SonomodeEdge> net) {

		// <- LOGGING
		if (logger.isDebugEnabled()) {
			logger.debug("Make " + numNewConnections + " new connections...");
		}
		// LOGGING ->

		int counter = 0;
		ArrayList<SonomodeAgent> remainingPotPartners = new ArrayList<SonomodeAgent>(
				net.numNodes());
		for (SonomodeAgent a : net.getNodes()) {
			remainingPotPartners.add(a);
		}
		remainingPotPartners.remove(agent);
		while (counter < numNewConnections && remainingPotPartners.size() > 0) {
			SonomodeAgent potPartner = remainingPotPartners.get(rand
					.nextIntFromTo(0, remainingPotPartners.size() - 1));
			if (!net.isSuccessor(potPartner, agent)) {
				net.connect(edgeMan.createEdge(net, potPartner, agent));
				// <- LOGGING
				if (logger.isDebugEnabled()) {
					logger.debug("Edge created: "
							+ net.getEdge(potPartner, agent));
				}
				counter++;
				// LOGGING ->
			}
			remainingPotPartners.remove(potPartner);
		}
	}
}
