/**
 * Center for Environmental Systems Research, Kassel
 * Created by holzhauer on 05.01.2009
 */
package de.cesr.sonomode.util;


import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.log4j.Logger;

import de.cesr.parma.core.PmParameterManager;
import de.cesr.parma.definition.PmFrameworkPa;



/**
 * 
 * @author holzhauer
 * @date 05.01.2009
 * 
 *       The class contains a number of routines that connect to the DB and fetch certain data.
 * 
 */
public class DataProvider {

	/**
	 * Logger
	 */
	static private Logger			logger	= Logger.getLogger(DataProvider.class);

	private static Connection		con;

	/**
	 * The instance of this class
	 */
	protected static DataProvider	instance;

	/**
	 * Retrieve parameter manager
	 */
	protected DataProvider() {
	}

	/**
	 * @return an instance of this class
	 */
	public static DataProvider getInstance() {
		if (instance == null) {
			instance = new DataProvider();
		}
		return instance;
	}



	/**
	 * Returns the current connection. If not existing, it tries to establish a JDBC Connection to the MySQL database
	 * given the settings provided to the constructor.
	 * 
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SQLException
	 */
	static protected Connection getConnection() throws InstantiationException, IllegalAccessException,
			ClassNotFoundException, SQLException {

		if (con == null || con.isClosed()) {
			Properties properties = new Properties();
			properties.put("user", PmParameterManager.getParameter(PmFrameworkPa.USER));
			properties.put("password", PmParameterManager.getParameter(PmFrameworkPa.PASSWORD));
			String connectTo = "jdbc:mysql://" + PmParameterManager.getParameter(PmFrameworkPa.LOCATION)
					+ (((String) PmParameterManager.getParameter(PmFrameworkPa.LOCATION)).endsWith("/") ? "" : "/")
					+ PmParameterManager.getParameter(PmFrameworkPa.DBNAME);

			// error handling:
			if (PmParameterManager.getParameter(PmFrameworkPa.LOCATION) == null) {
				throw new IllegalArgumentException("Invalid database settings: Invalid database location!");
			}
			if (PmParameterManager.getParameter(PmFrameworkPa.DBNAME) == null) {
				throw new IllegalArgumentException("Invalid database settings: Invalid database name!");
			}
			if (PmParameterManager.getParameter(PmFrameworkPa.USER) == null) {
				throw new IllegalArgumentException("Invalid database settings: Invalid user name!");
			}
			if (PmParameterManager.getParameter(PmFrameworkPa.PASSWORD) == null) {
				throw new IllegalArgumentException("Invalid database settings: Invalid password!");
			}

			Class.forName("com.mysql.jdbc.Driver").newInstance();
			con = DriverManager.getConnection(connectTo, properties);
		}
		return con;
	}

	/**
	 * Execute the given SQL statement and return the according ResultSet
	 * 
	 * @param sql the statement to query
	 * @return the ResultSet Created by Sascha Holzhauer on 13.07.2010
	 */
	public ResultSet connect(String sql) {

		try {
			
			// <- LOGGING
			if (logger.isDebugEnabled()) {
				logger.debug("Connection: " + getConnection().toString());
				logger.debug("SQL: " + sql);
			}
			// LOGGING ->
			
			if (sql.startsWith("update:")) {
				getConnection().createStatement().executeUpdate(sql.substring(7));
				return null;
			} else {
				return getConnection().createStatement().executeQuery(sql);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Disconnect the current mySQL connection
	 */
	public static void disconnect() {

		try {
			if (con != null && !con.isClosed()) {
				con.close();
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
